pro creategauss,flux,fwhmx,fwhmy,xtrunc,ytrunc,nx,ny,z
;
; creates a 2D gaussian array Z (real, dimensions NX * NY) with FWHM (pixel 
; units) FWHMX in X and FWHMY in Y, out to a maximum extent XTRUNC, YTUNC
; (pixel units). To get a gaussian centred at the mid point of the
; array NX and NY should be odd - this is especially recommended for
; small FWHM when the gaussian approaches a point source, which wuold
; be fundamentaly impossible to achieve if the source is centred on a
; pixel boundary in an even array. Z is scaled such that the integral
; over Z is FLUX. Z is set to zero either when it is less that exp(-explim)
; of the peak (10^10 dynamic range for explim=25) and/or when the
; spatial truncation limit is exceeded.
;
;print,'CREATEGAUSS INPUT:'
;print,'flux,fwhmx,fwhmy,xtrunc,ytrunc,nx,ny'
;print,flux,fwhmx,fwhmy,xtrunc,ytrunc,nx,ny
;
explim=25.0 ; sets gaussian to zero at dynamic range of 10^10
const=1.66511
;const=1.38629
;
; offset X in units of truncation distance
ccol=(findgen(nx)-float(nx/2))/xtrunc
;print,ccol
col=xtrunc*ccol*const/fwhmx
col=col*col
ii=where(col ge explim,nii)
if(nii gt 0)then col(ii)=explim
col=exp(-col) 
if(nii gt 0)then col(ii)=0.0
ii=where(abs(ccol) gt 1.0,nii)
if(nii gt 0)then col(ii)=0.0
;
; offset Y in units of truncation distance
rrow=(findgen(ny)-float(ny/2))/ytrunc 
row=ytrunc*rrow*const/fwhmy
row=row*row
ii=where(row ge explim,nii)
if(nii gt 0)then row(ii)=explim
row=exp(-row) 
if(nii gt 0)then row(ii)=0.0
ii=where(abs(rrow) gt 1.0,nii)
if(nii gt 0)then row(ii)=0.0
;
test=fltarr(nx)
z=fltarr(nx,ny)
for irow=0,ny-1 do begin
; offset from centre as ratio to elliptical truncation distance
test(*)=sqrt(rrow(irow)*rrow(irow)+ccol(*)*ccol(*)) 
ii=where(test le 1.0,nii)
if(nii gt 0)then z(ii,irow)=row(irow)*col(ii)
endfor ; irow
;
ii=where(z lt exp(-explim),nii)
if(nii gt 0)then z(ii)=0.0
if(nii eq nx*ny)then print,'CREATEGAUSS: warning: no points calculated'
z=flux*(z/total(z))
;
return
end
