pro dg1,otfstart,otfstop,sigclipdg1,niter,nglmin,nrelmin,k1,k2,pixplt
;
; for each pixel iteratively finds: 
; - relativisation factors in array rel (a + b*signal) 
;   (double precision) for each of the ninramp-1 read identifiers
; - noise (for each chopper plateau all reads are fitted with a polynomial after
;          relativistation and the rms of the residuals are found for each
;          readout class. Then a running median and convolution is used to find
;           the noise. This noise rms is taken into account in the fit of the 
;          polynomial in the next iteration)
; - bad   (data outside sigclipdg1 * rms of fit are set to bad = 1)
;
;         in the iterative scheme only bad=0 from previous iteration are used 
;         current iteration to find relativisation and rms. Bad is reset to 
;         badin (bad as set by telemetry flags and the no-cross-readout 
;         limitation) after each iteration. 
; - glitch (array containing glitch strengths)
;
; input:  nptin: number of reads in dataset per pixel
;         npix: number of pixels
;         niter: number of iterations to use
;         ninramp: number of reads in ramp (ninramp-1 good pairs)
;         ny,nz: raster dimension
;         sigclipdg1: no. of rms for a data point to be called bad
;         otfstart(ny,nz),otfstop(ny,nz) effective range of stable pointing
;         badin(npix,nptin) bad data from telemetry flags, or ndr - dr
;         s_in(npix,nptin) Vs-1 (=V(i+1)-V(i))
;         rs(nptin) readout cycle (eg  1 2 3 4 1 .... for ninramp=4)
;         cs(0:nch) gives the chopper transition reads
;         nch there are nch+2 chopper transitions
;         k1 and k2 are print/plot range for diagnostic output
;         pixplt is pixel for diagnostics (normally = plotted pixel in calling
;         routine).
;         irelstart: the relativisation is scaled to the 
;         average signal from the read classes IRELSTART....NINRAMP-IRELSTART
;         the first signal (V/s) is 1 and the last NINRAMP-2. Typically for 
;         a P32 with ninramp=4 IRELSTART should be 2 but a P40 raster
;         might differ.
;         NRELMIN is the minimum reads of each read class on a chopper plateau
;         for the chopper plateau to be included in the relativisation
;         and rms calculation
;  nglmin  minimum number of surviving points on a ramp to calculate
;          signal after deglitching in glitch1 routine. All points set bad if
;          number of points surviving within specified statistical tolerance
;          (sigclipdg1) is < nglmin
;
; output: (all common block)
;         bad: =0 for good data =1 for bad data
;         rel(npix,2,ninramp-1): relativisation factors scorr=a+b*s
;         rms(npix,nptin): pixel, readout and time dependent rms determined 
;         from running median 
;         glitch(npix,nptin): glitch strengths
;         s(npix,nptin) Vs-1 (=V(i+1)-V(i)) deglitched and relativised
;
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;
@common_steerin
@common_steer1
@common_signal
;
;print,' '
kk2=abs(k2) ; printing parameters
kk1=abs(k1)
n=nptin-1 ; number of data points in s, rs, bad etc (note dimensions are
;           made ninramp not ninramp-1 because otherwise where idl function 
;           might get confused).
bad=badin 
rel=dblarr(npix,2,ninramp-1)
rms=dblarr(npix,nptin)
glitch=rms
;s=s_in already done in sigcreate
;
iter=0L
c555: ; major iteration loop
iter=iter+1
print,'DG1: iteration',iter
badprev=bad
bad=badin
;
; find relativisation factors (array rel) ignoring
; bad data points in badprev and update s for these factors
;
relativise,badprev,nrelmin,k1,k2,pixplt
;rel(*,0,*)=0.0D ;*** to bypass relativisation
;rel(*,1,*)=1.0D ;*** to bypass relativisation
;s=s_in ;*** to bypass relativisation
;
; find rms, suitably smoothed over range of data
;
findrms,badprev,nrelmin
;
; find glitches and update bad
;
glitch1,sigclipdg1,nglmin
;
if(iter ge niter)then goto, c600
goto, c555
c600:
;
return
end
; 
