;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       ERD2MAP_MSG          
;
; PURPOSE:
;       This routine pops up a status message window. 
;
; DESCRIPTION:
;       A model widget is created displaying the input text message.  
;
; CALLING SEQUENCE:
;       erd2map_msg, msg, GROUP=group, TITLE=title  
;
;INPUTS:
;       msg:      string message to be displayed
;
; KEYWORDS:
;       GROUP:    Group id of parent widget
;       TITLE:    title on the window. "Acknowledge" is default
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;           
; CALLED ROUTINES:
;       None
;
; CALLED FROM:
;       PIA_ERD2MAP_INIT

; COMMON BLOCKS:
;       None        
;
; PROCEDURE:
;       None        
;
; SIDE EFFECTS:
;       None
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;       original design                        Aug 17, 2001
;-
;-----------------------------------------------------------------------

;******************************************************************************

pro erd2map_msg_event,ev
widget_control,ev.top,/destroy				; destroy the modal widget
end

;*****************************************************************************

pro erd2map_msg, msg,        $			; text to display
		GROUP=group, $			; top level base to orient to
		TITLE = title                   ; title on the window




; set up title, if none given
if not keyword_set(title) then title = 'Acknowledge'

; create the base

  ack_base = widget_base(/column,title = title, /modal, GROUP=group)	; create the base
; find the longest string in the input
  xsiz = max(strlen(msg))
  siz = size(msg)	
; if it is an array, use num of dims, else use 1
  if siz(0) gt 0 then ysiz = siz(1) else ysiz = 1	
   junk = widget_text(ack_base,xsize = xsiz,ysize = ysiz) 
; make a nice base for the buttons
   horiz_base = widget_base(ack_base,/row)
   acceptwid = widget_button(horiz_base,value = "  OK  ",uval = 'accept')

widget_control,ack_base,/realize						; realize it
widget_control,junk,set_value=msg  
widget_control,acceptwid,/input_focus
xmanager,'erd2map_msg',ack_base
end

