pro erdplot,psout,n1,n2,pixplt
;
; interactively plots idl data just processed by erdtomap
;
; if psout is 0 or negative, then the single window -psout is opened
; for plotting by running pr_open, pr_close to get plot.
; if psout is 999 all plots can be psed via pr_open
;
; example run lines from IDL
; idl
; .run USR:[RJT.IDL.proG.ERDTOMAP]erdplot
; erdplot,1,0,30,5 ; to get all windows on screen for pixel 5 between 
;                    reads 0 30
;
; n1 and n2 are points for range of diagnostic plotting n1:n2. If n1 and n2
; are negative then -n1:-n2 refers to a range of fine pointings
; eg erdplot,1,-1,-1,3 = first raster point for pixel 3
; if n1 not same sign as n2 than plots are against time between n1 and n2
; The otf is defined by the original ISO input
;
;COMMON PHTERD, PHTERD, PHTERD_REC
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed
;common timer,time
;common const,pie
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
;
@common_phterd
@common_steerin
@common_steer1
@common_signal
@common_timer
@common_const
@common_plotter
;
; program constants
;
print,'number of points in data = ',nptin
print,'psout = ',psout
yss=200.0 ; plotting constant
;
print,'n1 n2 plot range: ',n1,n2
print,'pixplt: ',pixplt
;
; plot chopper, data, pointing etc. 
;
; now set up plotting ranges for case n1 n2 negative (is so only points
; in range otfstart otfstop are plotted
;
if(n1 lt 0 and n2 lt 0)then begin
nn1=-n1-1 ; to plot raster points in range nn1 to nn2
nn2=-n2-1
startendpoint,nn1,nn2,irasterstart,irasterstop,n1,n2,$
irasterstarte,irasterstope,istartexpand,istopexpand
;n1=otfstart(nn1,nn2)
;n2=otfstop(nn1,nn2)
endif
print,'data will be plotted for readout range n1 n2 = ',n1,n2
itimplot=0
xplot=rd
help,xplot
xxran=[n1,n2]
if(n1 lt 0 and n2 gt 0)then begin ;plot against time
itimplot=1
n1=-n1
xplot=time
xxran=[time(n1),time(n2)]
help,xplot
endif
if(n1 ge 0 and n2 lt 0)then begin ;plot against time
itimplot=1
n2=-n2
xplot=time
xxran=[time(n1),time(n2)]
endif
;
print,'n1 n2 = ',n1,n2
print,'*** xxran',xxran
;
iplt=where(rdstat ne 1 and rd(*) gt n1-1 and rd(*) lt n2+1) ; for pos
igplt=where(rd(*) gt n1-1 and rd(*) lt n2+1 and bad(pixplt-1,*) eq 0)
;
if(psout gt 0 and psout ne 999)then window,0,title='chop and read status, flags',$
xs=400,ys=yss,xpos=430,ypos=1000-(yss+20)
if(psout gt 0 or psout eq 0)then begin 
print,'plotting window 0:'
plot,xplot(n1:n2),chop(n1:n2),yran=[-100,250],xstyle=1,ystyle=1,xtickl=1
oplot,xplot(n1:n2),rdstat(n1:n2)*40+100
oplot,xplot(iplt),badin(pixplt-1,iplt)*40+150
oplot,xplot(iplt),bad(pixplt-1,iplt)*40+200
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,3,title='otf',$
xs=400,ys=yss,xpos=10,ypos=1000-(yss+20)
if(psout gt 0 or psout eq -3)then begin 
print,'plotting window 3:'
plot,xplot(n1:n2),otf(n1:n2),yran=[-1.5,1.5],xstyle=1,xticklen=1
endif ; psplot
;
;help,rpid
;
if(psout gt 0 and psout ne 999)then window,4,title='rpid',$
xs=400,ys=yss,xpos=10,ypos=1000-(yss+20)*2.0
if(psout gt 0 or psout eq -4)then begin 
print,'plotting window 4:'
plot,xplot(n1:n2),rpid(0,n1:n2),yran=[min([rpid(0,n1:n2),rpid(1,n1:n2)])-0.5,$
max([rpid(0,n1:n2),rpid(1,n1:n2)]+0.5)],xstyle=1,xticklen=1
oplot,xplot(n1:n2),rpid(1,n1:n2),linestyle=1
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,1,title='s/c offset Y from map centre (arcsec)',$
xs=400,ys=yss,xpos=10,ypos=1000-(yss+20)*3
if(psout gt 0 or psout eq -1)then begin 
print,'plotting window 1:'
help,yrel
print,'yrel(n1),yrel(n2),n1,n2',yrel(n1),yrel(n2),n1,n2
plot,xplot(n1:n2),yrel(n1:n2),xstyle=1,ystyle=1,xticklen=1
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,2,title='s/c offset Z from map centre (arcsec)',$
xs=400,ys=yss,xpos=10,ypos=1000-(yss+20)*4
if(psout gt 0 or psout eq -2)then begin 
print,'plotting window 2:'
plot,xplot(n1:n2),zrel(n1:n2),xstyle=1,ystyle=1,xticklen=1
endif ; psout
;
pistr=['1','2','3','4','5','6','7','8','9']
tit7=strarr(npix,20)
tit8=strarr(npix,17)
tit9=strarr(npix,11)
tit5=strarr(npix,18)
tit6=strarr(npix,18)
tit10=strarr(npix,26)
for i=0,npix-1 do begin
tit8(i)='glitches pixel '+pistr(i)
tit9(i)='rms pixel '+pistr(i)
tit7(i)='input signal pixel '+pistr(i)
tit5(i)='Y position pixel '+pistr(i)
tit6(i)= 'Z position pixel '+pistr(i)
tit10(i)='deglitched signal, pixel '+pistr(i)
endfor
ip=lonarr(ninramp-1)
;
if(psout gt 0 and psout ne 999)then window,7,title=tit7(pixplt-1),xs=400,ys=yss,xpos=430,ypos=1000-(yss+20)*2
if(psout gt 0 or psout eq -7)then begin 
print,'plotting window 7:'
for i=0,ninramp-2 do begin
ip=where(rdstat ne 1 and rs(*) eq i+1 and $
rd(*) gt n1-1 and rd(*) lt n2+1) ; for different non-destructive reads
if(i eq 0)then $
plot,xplot(ip),s_in(pixplt-1,ip),xstyle=1,ystyle=1,$
yran=[min(s_in(pixplt-1,iplt)),max(s_in(pixplt-1,iplt))],xran=xxran,xtickl=1
if(i gt 0)then $
oplot,xplot(ip),s_in(pixplt-1,ip),linestyle=i
endfor
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,8,title=tit8(pixplt-1),xs=400,ys=yss,xpos=850,ypos=1000-(yss+20)*3
if(psout gt 0 or psout eq -8)then begin 
print,'plotting window 8:'
plot,xplot(iplt),glitch(pixplt-1,iplt),xstyle=1,ystyle=1,$
yran=[min(glitch(pixplt-1,iplt)),max(glitch(pixplt-1,iplt))],$
xran=xxran,xtickl=1
dd=max(glitch(pixplt-1,iplt))-min(glitch(pixplt-1,iplt))
;oplot,xplot(n1:n2),bad(pixplt-1,n1:n2)*0.2*dd+0.8*dd+min(s(pixplt-1,iplt))
;oplot,xplot(iplt),rms(pixplt-1,iplt),linestyle=1
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,9,title=tit9(pixplt-1),xs=400,ys=yss,xpos=850,ypos=1000-(yss+20)*2
if(psout gt 0 or psout eq -9)then begin 
print,'plotting window 9:'
plot,xplot(iplt),rms(pixplt-1,iplt),xstyle=1,ystyle=1,$
yran=[min(rms(pixplt-1,iplt)),max(rms(pixplt-1,iplt))],xran=xxran,xtickl=1
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,5,title=tit5(pixplt-1),xs=400,ys=yss,xpos=430,ypos=1000-(yss+20)*3
if(psout gt 0 or psout eq -5)then begin 
print,'plotting window 5:'
plot,xplot(iplt),ytrue(pixplt-1,iplt),xstyle=1,ystyle=1,xran=xxran,xtickl=1
endif ; psout
if(psout gt 0 and psout ne 999)then window,6,title=tit6(pixplt-1),xs=400,ys=yss,xpos=430,ypos=1000-(yss+20)*4
if(psout gt 0 or psout eq -6)then begin
print,'plotting window 6:'
plot,xplot(iplt),ztrue(pixplt-1,iplt),xstyle=1,ystyle=1,xran=xxran,xtickl=1
endif ; psout
;
if(psout gt 0 and psout ne 999)then window,10,title=tit10(pixplt-1),xs=400,ys=yss,xpos=850,ypos=1000-(yss+20)
if(psout gt 0 or psout eq -10)then begin 
print,'plotting window 10:'
maxs=max(s(pixplt-1,igplt))
mins=min(s(pixplt-1,igplt))
diffs=maxs-mins
plot,xplot(igplt),s(pixplt-1,igplt),xstyle=1,ystyle=1,$
;yran=[min(s(pixplt-1,igplt)),max(s(pixplt-1,igplt))],xran=xxran,xtickl=1
;yran=[mins,(maxs+0.2*diffs)],xran=xxran,xtickl=1
yran=[mins,maxs],xran=xxran,xtickl=1
;dd=max(s(pixplt-1,igplt))-min(s(pixplt-1,igplt))
;oplot,xplot(n1:n2),gbad(pixplt-1,n1:n2)*0.1*diffs+0.9*diffs+min(s(pixplt-1,igplt))
maxch=max(chop(n1:n2))
minch=min(chop(n1:n2))
mchdiff=maxch-minch
if(maxch gt minch)then oplot,xplot(n1:n2),$
(((chop(n1:n2)-minch)*0.2*diffs/mchdiff)+(0.8*diffs)+$
min(s(pixplt-1,igplt))),linestyle=0
endif ; psout
;
return
end
