pro erdtoscp,ipix
;
; fill SCPBLOCK common block with data-dependent arrays from unexpanded data 
; for pixel IPIX. SCPBLOCK must have already been dimensioned by a previous 
; previous call to INITSCP eg from ERDTOMAP) and the data-independent arrays 
; filled there.
;
; input from ERD level from common SIGNAL:  
;     S(npix,*) data
;     BAD(npix,*) equals 0 for good data
;     RMS(npix,*) rms of individual ERD data sample
;
; input from SCP level from common SCPBLOCK:
;     KPT_SCP(npix,nplat_scp)pointing direction on P32 natural grid of 
;                            fine pointing direction. For a slew it is the
;                            most common pointing direction of the good points
;                            on the plateau
;     VIG_SCP(nplat_scp)     vignetting factors for each chopper plateau
;     SPT_SCP(nplat_scp)     s/c pointing identifyer (in time order). For a 
;                            slew it is set to -1
;     FPT_SCP(nplat_scp)     =1 if all good or bad points on the plat are on a
;                            fine pointing, =0 otherwise
;     FROM_SCP(nplat_scp)    ERD ident of 1st point of plateau (even if bad)
;     TO_SCP(nplat_scp)      ERD ident of last point of plateau (even if bad)
;     TIME_SCP(nplat_scp)    Time of first sample in each plateau 
;                            (for simplicity) 
;     NPLAT_SCP              Number of chopper plateaus in the SCPBLOCK arrays
;
; output to common SCPBLOCK:
;     S_SCP(npix,nplat_scp)  = mean of all good points on plateau
;                            derived from input S array  
;     RMS_SCP(npix,nplat_scp)= error in mean of all good points on plateau
;                            derived from input RMS array  
;     BAD_SCP(npix,nplat_scp)=0 if at least one input ERD sample on plateau iss
;                            good =1 otherwise
;     N_SCP(npix,nplat_scp)  number of good elementary ERD samples contributing
;                            to plateau
;
@common_signal
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_scpblock
;common scpblock,s_scp,bad_scp,rms_scp,n_scp,kpt_scp,spt_scp,fpt_scp,$
;from_scp,to_scp,time_scp,vig_scp,nplat_scp
;
npt=n_elements(s(0,*)) ; not actually used
npix=n_elements(kpt_scp(*,0)) ; not actually used
;
for iplat=0,nplat_scp-1 do begin
;
i1=from_scp(iplat)
i2=to_scp(iplat)
ninplat=i2-i1+1
;
igood=where(bad(ipix,i1:i2) eq 0,nigood)
if(nigood le 0)then begin
s_scp(ipix,iplat)=0.0
n_scp(ipix,iplat)=0
bad_scp(ipix,iplat)=1
rms_scp(ipix,iplat)=0.0
endif ; nigood le 0
;
if(nigood gt 0)then begin
s_scp(ipix,iplat)=total(s(ipix,i1+igood))/float(nigood)
bad_scp(ipix,iplat)=0
n_scp(ipix,iplat)=nigood
dum=s(ipix,i1+igood)
dumrms=rms(ipix,i1+igood)
imethod=0 ; equal weights for all data
rmsprop,dum,dumrms,xmean,xsig,xsigmean,imethod
rms_scp(ipix,iplat)=xsigmean
endif ; nigood gt 0
;
endfor ; iplat=0,nplat_scp-1
;
return
end









