pro findbestsky,ifinemethod,kfine,$
nskyfound,skyfound,skyfoundfitres,skyfoundnpt,ngrid,bestsky,nbestsky
;
; find updated best guess for the best guess sky on the fine pointings KFINE
; BESTSKY(kfine) (where 0 LE KFINE LE NGRID-1) from accumulated 
; previous results using
; information in SKYFOUND, SKYFOUNDFITRES, SKYFOUNDNPT. 
;
; input:
;         KFINE       integer array containing identifiers of P32 natural
;                     map grid points for which BESTSKY is to be updated
;         NGRID       number of map pixels on the P32 natural grid
;                     (NGRID = NYGRID * NZGRID)
;             SKYFOUND (floating array dimension ngrid,ndim in V/s) where 
;                      points in 0:ndim-1 are only filled from 0 to 
;                      NSKYFOUND-1. This is the previous
;                      best estimate of illumination on each fine pointing
;                      position on the P32 natural grid converted to 1D.
;       SKYFOUNDFITRES estimate of quasi RMS from calls to SOLVEILLUM1,
;                      corresponding to SKYFOUND
;          SKYFOUNDNPT Number of elementary data samples used in the fitting
;                      to obtain each element of SKYFOUND (dimensions
;                      correspond to SKYFOUND)
;            NSKYFOUND (array dimension NGRID). Number of pointing plateaus
;                      with fine pointing flag set for each fine pointing
;                      position in the SKYFOUND... arrays.   
;          IFINEMETHOD (Integer Flag)
;                      Determines method to get the illumination
;                      on fine pointings. 
;                  = 1 mean
;                  = 2 weighted mean (using skyfoundnpt as weight)
;                  = 3 weighted mean (using 1/(skyfoundfitres)^2 as weight)
;                  = 4 median (using MEDIANTRUE routine)
;                               
; output: BESTSKY      (1D array in V/s dimension NGRID) 
;                      This is the illumination that each 
;                      point on the natural sky grid would receive in the
;                      absence of vignetting. The dimension of sky is the
;                      total number of points on the natural P32 grid (NGRID)
;                      BESTSKY is left zero where there are no contributing 
;                      data points. It is calculated from the accumulated 
;                      information in the input SKYFOUND series of arrays.
;         NBESTSKY     (ID integer array) Total number of elementary 
;                      data samples contributing to the value for BESTSKY.
;                      Note that FINDILLHIST always tries to find plausible
;                      values for BESTSKY even for directions for which
;                      NBESTSKY is zero.
;         BESTRMS      (common RMS_IMP) (1D real array in V/s dimension NGRID) 
;                      equivalent RMS on each sample which would reproduce
;                      the dispersion in drift solutions for a P32 grid 
;                      direction. Set to zero if a P32 grid direction
;                      measured less than twice.
;
@common_rms_imp
;common rms_imp,bestrms,bestsolrms
;
; Find best guess for sky on natural map pointing directions. 
; If any natural point has no data bestsky is set to 0.0 Note that since 
; the optimisation in the calling routine only admits positive values
; bestsky LE 0.0 can be taken as a sign that it contains no data for that
; data point.
;
;igridfound=where(nskyfound gt 0,nigridfound)
;
igridfound=[kfine]
;
; RJT 06JUL00
;
ii=where(nskyfound(kfine) gt 0,nigridfound)
if(nigridfound le 0)then begin
print,'FINDBESTSKY *** warning - no measurements found for specified pointings'
goto,c999
endif ; nigridfound le 0
igridfound=kfine(ii)
;
; end RJT 06JUL00
;
;
; following commented RJT 06JUL00
;
;if(n_elements(kfine) eq ngrid)then begin ; if all natural grid specified 
;                                  only calculate for grid points with data 
;                                  in SKYFOUND 
;igridfound=where(nskyfound gt 0,nigridfound)
;endif ; n_elements(kfine) eq ngrid
;
nigridfound=n_elements(igridfound)
;
; end comments RJT 06JUL00
;
if(nigridfound gt 0)then begin
;
for ig=0,nigridfound-1 do begin
igrid=igridfound(ig)
;
;if(nskyfound(igrid) gt 0)then begin ; data exists for this fine pointing
; (deal with points with no data on plateau in plateau by plateau loop below)
; leave bestsky unassigned if there are no data points.
;
n=nskyfound(igrid)
; we should never call FINDBESTSKY if there are no data points available
if(n le 0)then print,$
'FINDBESTSKY: warning - no points found for igrid = ',igrid
test=fltarr(n)
test(*)=skyfound(igrid,0:n-1)
nbestsky(igrid)=n
testrms=sqrt(skyfoundnpt(igrid,0:n-1)) 
; RMSPROP weights mean and rms calculations with (1/testrms)^2 with IMETHOD=1
rmsprop,test,testrms,testmean,testsig,testsigmean,1
if(n gt 1)then bestrms(igrid)=testsig*sqrt(float(n-1)) ; actual weighted rms ..
if(n eq 1)then bestrms(igrid)=0.0                      ; .. dispersion of plats
; then find equivalent rms for each elementary sample
bestrms(igrid)=bestrms(igrid)*sqrt( total(skyfoundnpt(igrid,0:n-1))/float(n) )
;
if(ifinemethod eq 1)then begin ; means
bestsky(igrid)=total(test)/float(n)
endif ; ifinemethod eq 1
;
if(ifinemethod eq 2)then begin ; weighted mean (n)
w=test
w(*)=float(skyfoundnpt(igrid,0:n-1))
;w=sqrt(w)
sumw=total(w)
bestsky(igrid)=total(w*test)/sumw
endif ; ifinemethod eq 2
;
if(ifinemethod eq 3)then begin ; weighted mean (rms)
w=test
nw=test
w(*)=float(skyfoundfitres(igrid,0:n-1))
nw(*)=float(skyfoundnpt(igrid,0:n-1))
iii=where(nw gt 0.0,niii)
if(niii gt 0)then begin
w=w(iii)
nw=nw(iii)
test=test(iii)
w=nw/w
sumw=total(w)
endif ; niii gt 0
if(niii le 0)then begin
w(*)=1.0
sumw=total(w)
endif ; niii le 0
bestsky(igrid)=total(w*test)/sumw
endif ; ifinemethod eq 3
;
if(ifinemethod eq 4)then begin ; median
mediantrue,test,testmed
bestsky(igrid)=testmed
endif ; ifinemethod eq 4
;
if(bestsky(igrid) le 0.0)then begin ; should not happen if solveillum1
;                                     only admits positive solutions
print,'FINDBESTSKY: warning - igrid, bestsky(igrid) = ',igrid,bestsky(igrid) 
print,'... skyfound(igrid:0:n-1) = ',skyfound(igrid,0:n-1)
print,'... skyfoundnpt(igrid:0:n-1) = ',skyfoundnpt(igrid,0:n-1)
;
endif ; bestsky(igrid) le 0.0
;
endfor ; ig=0,nigridfound-1
;
endif ; nigridfound gt 0
;
c999:
;
return
end






