pro findbetataufcsillum,pixarray,imodel,isquare,ibefore,sillumprev,$
betadown,betaup,taudown,tauup,betaout,tauout,sillumout,$
fraction,imedfilter,fitresultout
;
print,'calling FINDBETATAUFCSILLUM with parameters'
print,'pixarray,imodel,isquare,ibefore,sillumprev,fraction,imedfilter = '
print,pixarray,imodel,isquare,ibefore,sillumprev,fraction,imedfilter
print,'... betadown,betaup,taudown,tauup = ',betadown,betaup,taudown,tauup
;
; For two free parameter (beta, tau) detector model specified by IMODEL and a
; further free parameter sillumout for the fcs illumination find best 
; fits to detector response to sudden fcs illuminations at start or end of a 
; P32 observation in a particular filter, in each pixel in PIXARRAY. It is 
; assumed that the detector is in equilibrium prior to the fcs illumination, 
; with response=illumination from the sky viewed at the central chopper 
; position at the first or last s/c raster position. We note that this 
; assumption may be more readily met for the first fcs illumination for the 
; C100 detector due to the 240s stabilisation time, when the sky has strong 
; structure. FINDBETATAUFCSILLUM should normally be called at the second and 
; subsequent
; iterations of DRIFTCORR (ie after the first good prediction of sky 
; illumination is available). The fits are displayed on the screen. A warning
; is given if the paraneters found are at the boundaries of the search range.
;
; The analysis is done from imported SRD 'internal PIA' datasets which should
; already have been corrected at the ERD stage for first stage deglitch, 
; non-linearity, linear ramp fit, standard selection (ignores 1st and 
; destructive read) and at the SRD stage for deglitchsecond stage, reset 
; correction, dark, responsivity drift recognition. Note that the 
; responsity drift recognition was done to get a first reasonable guess
; for the responsitvity for the target in the SCP to SPD processing
; (accessed by CAL32 in the ERDTOMAP suite). This is retained here since 
; all fcs and processed in parallel to the target fields in the PIA processing
; prior to ERDTOMAP but has no effect on the processing in this subroutine.
; 
; If the values for SILLUMOUT are close to those for SILLUMIN a warning
; is issued, since, without a flux step there can be no possibility 
; of finding beta and tau.
;
; The analysis proceeds by finding best fit for fcs illumination for trial 
; values of beta (binary chop) and tau (binary chop in called FINDTAUILLUM 
; routine).
;
; Note. We assume a single time constant - not a fast and a long response.
; Therefore there should be no need to process at the ERD resolution.
;
; input (common NAMES):
;    TDTNAM: string denoting name of TDT eg '53801001'
;    ISEQ:   string denoting sequence number of target source map
;            (normally '01' or '04' or '07' for multifilter sequence separated
;             by FCS measurements) 
;    DIRNAM: string denoting irectory name for internal PIA files
;            eg '/d13/vlk/rjt/pia/p32p22p25/v7_3/'
;  input:
;    PIXARRAY = (integer*NPIXLOOP) array of pixels to correct for 
;               (useful for diagnostics
;    ARRAY   (common STEERIN) = 'c100' or 'c200'
;    IBEFORE = 1 to process preceding FCS illumination 
;            = 0 to process succeeding FCS illumination
;         BETADOWN,BETAUP ; search range for beta
;         TAUDOWN,TAUUP ; search range for tau
;    SILLUMPREV (1D array in V/s dimension NPIXLOOP):
;            illumination immediately
;            prior to fcs illumination at start or end of observation  
;            (found in calling routine DRIFTCORR). (Note for succeedinf fcs
;            measurement this will depend on whether the NZ is even or odd).
;         IMEDFILTER   if not 0 applies mean filter to FCS data, using AVRED
;                      routine with width 
;                      about 10 percent of data stream. (NB this should only
;                      be done when the actual time constant (illum/tau for
;                      Foukes Schubert  imodel=1) is longer than ca. 1/10 at 
;                      least of the FCS data stream. Median filtering
;                      improves results for glitchy data with long time 
;                      constants. For glitchy data with short time constants
;                      It is best to run FINDBETATAUFCSILLUM with a truncated 
;                      data set (FRACTION < 1.0) and use ISQUARE=2.
;         FRACTION     0 < FRACTION <= 1 Only the first fraction FRACTION
;                      of the data is used in the fit. Gives improved results
;                      for fast detector response when the long term response
;                      is controlled by a long term responsivity drift
;                      independent of the short term effects. 
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;
; output:
;         BETAOUT (1D array dimension NPIXLOOP): first 
;                     detector model constant for pixels specified in PIXARRAY 
;                     derived from fcs illumination at start or end of 
;                     observation.
;         TAUOUT  (1D arrays in VOLTS dimension NPIXLOOP): second 
;                     detector model constant for pixels
;                     specified in PIXARRAY derived from fcs illumination at 
;                     start or  end of observation.
;         SILLUMOUT (1D array dimension NPIXLOOP in V/s): best 
;                     fit constant 
;                     illumination for each pixel specified by PIAXARRAY from
;                     fcs illumination at start or end of observation.
;         FITRESULTOUT (1D array dimension NPIXLOOP) goodness of fit 
;                     - lower is better - according to
;                     value of input parameter ISQUARE - see description
;                     of isquare, above)
;
; output to common block FCSCAL: (these results will be needed by CAL32 to 
;                     correct the "responsivity drift recognition" result
;                     for residual systematic errors due to the transient 
;                     response of the fcs). If no results for any pixel have 
;                     previouslyb een obtained by FINDBETATAUFCSILLUM then all
;                     the following arrays are first initialised with zeros.   
;         FOUNDBEFORE,FOUNDAFTER - (1D integer arrays V/s dimension NPIX where
;                     NPIX is total number of detector pixels)
;                     =1 if an fcs fit has been performed and the results
;                     stored in the arrays SILLUMPREVBEFORE, SILLUMPREVAFTER,
;                     ILLBEFORE ILLAFTER FOUNDBEFORE FOUNDAFTER
;                     =0 if an fcs fit has been performed. 
;                     The two arrays are for the cases before 
;                     and after the raster.
;         SILLUMBEFORE,SILLUMAFTER - (1D real arrays V/s dimension NPIX) fcs 
;                     illumination in V/s found by fitting routine for cases 
;                     before and after the raster.
;         FITENDBEFORE,FITENDAFTER - (1D real arrays V/s dim. NPIX). Value of
;                     Fit to FCS plateau at end of plateau (ie at time of last
;                     SRD plateau).
;         SILLUMPREVBEFORE, SILLUMPREVAFTER - (1D real arrays V/s dim. NPIX)
;                     Illumination before FCS switched on for cases FCS before 
;                     and after the raster. (Copied from input parameter
;                     SILLUMPREV).
;
;common names,tdtnam,iseq,dirnam,caldir,cfilter
@common_names
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_steerin ; NB don't use nptin,npt,ninramp,imeas in this routine!
;common fcscal,foundbefore,foundafter,sillumbefore,sillumafter,$
;fitendbefore,fitendafter,sillumprevbefore,sillumprevafter
@common_fcscal
;
; internal constants controlling trade off between speed, precision and 
; robustness of fit
;
if(array eq 'c100')then begin
;betamin=0.1 ; search range for beta
;betamax=0.99 ; must be < 1.0
taumin=5.0 ; search range for tau
taumax=100.0 
endif ; array eq 'c100'
;
if(array eq 'c200')then begin
;betamin=0.5 ; search range for beta
;betamax=0.99 ; must be < 1.0
taumin=5.0; search range for tau
taumax=100.0 
endif ; array eq 'c200'
;
betamin=betadown
betamax=betaup
taumin=taudown
taumax=tauup
;
if(fraction le 0.0)then begin
print,'FINDBETATAUFCSILLUM - aborting since input parameter FRACTION = ',$
fraction
goto,c999 
endif ; fraction le 0.0
;
; control parameter for precision of fit in this routine and the called
; routines FINDTAUILLUM and SOLVEILLUM1:
;
fitprec=0.1 ; for FITPREC * RMSarray is precision in SOLVEILLUM1
;betaprec=0.01 ; precision in beta for this roution
;tauprec=0.1 ; (Volts) precision in tau for FINDTAUILLUM routine
betaprec=0.01
tauprec=1.0
;
; control parameters for the cut in beta in this  routine:
niter=1 ; number of iterations in this routine 
;         for diagnostics - set to a large number to control convergence
;         by BETAPREC. In fact, though iter is bbuilt in to the logic, we
;         ignore it to avoid all possibility of getting a too crude
;         sampling of the minimum.
ncut=5 ; must be odd and at least 5
reduce=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
irmswk=1 ; to tune fitprec in FINDTAUILLUM call to the search interval
;              in tau (for speed). Note a corresponding parameter (not passed)
;              also operates in FINDTAUILLUM. In fact this does not speed 
;              things
;              up much but can make the routine unreliable. Subsequent tests
;              showed this effect was due to the absoluet limit for fitprecwk 
;              being set to 1.0. It should really be a maximum of 0.5 for
;              isquare=1.
;
;
TF2=2.0 ; time between FCS illumination and start of data taking
;
;
; read in data rms time flag for first and second fcs illumination to 
; fsig1, fsig2; frms1, frms2; t1, t2; flag1, flag2.
;
if(iseq eq '01')then begin
if(ibefore eq 1)then iseq1='00'
if(ibefore ne 1)then iseq1='02'
endif ; iseq eq '01'
if(iseq eq '04')then begin
if(ibefore eq 1)then iseq1='03'
if(ibefore ne 1)then iseq1='05'
endif ; iseq eq '04'
if(iseq eq '07')then begin
if(ibefore eq 1)then iseq1='06'
if(ibefore ne 1)then iseq1='08'
endif ; iseq eq '07'
if(iseq eq '10')then begin
if(ibefore eq 1)then iseq1='09'
if(ibefore ne 1)then iseq1='11'
endif ; iseq eq '10'
if(iseq eq '13')then begin
if(ibefore eq 1)then iseq1='12'
if(ibefore ne 1)then iseq1='14'
endif ; iseq eq '13'
if(iseq eq '16')then begin
if(ibefore eq 1)then iseq1='15'
if(ibefore ne 1)then iseq1='17'
endif ; iseq eq '16'
if(iseq eq '19')then begin
if(ibefore eq 1)then iseq1='18'
if(ibefore ne 1)then iseq1='20'
endif ; iseq eq '19
if(iseq eq '22')then begin
if(ibefore eq 1)then iseq1='21'
if(ibefore ne 1)then iseq1='23'
endif ; iseq eq '22'
;
if(array eq 'c100')then begin
isrd1 = dirnam+'C1'+iseq1+tdtnam+'.ISRD'
endif ; array eq 'c100'
if(array eq 'c200')then begin
isrd1 = dirnam+'C2'+iseq1+tdtnam+'.ISRD'
endif ; array eq 'c200'
print,'FINDBETATAU: reading SRD file (FCS)',isrd1
;
@phtsrd
read_isrd,isrd1
imeas=N_ELEMENTS(phtsrd) -1
handle_value,phtsrd(imeas).slop,fsig1 ; signal array (npix+2,n1)
; note only the first npix elements of fsig1, fsig1(0:npix-1) contain data
handle_value,phtsrd(imeas).slun,frms1 ; signal uncertainty (npix+2,n1)
handle_value,phtsrd(imeas).stim,t1 ; time (n1)
;handle_value,phtsrd(imeas).temp,temp ; temperature (n1)
;plot,t1,temp,yran=[min(temp),max(temp)],xsty=1
;plot,temp,fsig1(0,*),xran=[min(temp),max(temp)],xsty=1
t1=t1-t1(0)+t1(1)-t1(0) ; needed for SOLVEILLUM1 and handling of FRACTION
t1=t1+TF2
handle_value,phtsrd(imeas).flag,flag1 ; flag (n1)
n1=n_elements(fsig1(0,*)) ; number of data points
;
; process each pixel
;
npixloop=n_elements(pixarray)
betaout=fltarr(npixloop)
tauout=betaout
sillumout=betaout
fitresultout=betaout
signalendout=betaout
f1=lonarr(n1)
;
if(ibefore eq 1)then iunit=0 ; counter for output plots
if(ibefore ne 1)then iunit=10
loadct,15 ; stern special
silminlim=0.0001 ; minimum tolerated fcs signal
;
for pix=0,npixloop-1 do begin
;
ipix=pixarray(pix)
;
; plot fcs data for pixel ipix at start of observation
;
if(ipix eq 0)then tit=tdtnam+' '+iseq1+' FCS PIXEL 0'
if(ipix eq 1)then tit=tdtnam+' '+iseq1+' FCS PIXEL 1'
if(ipix eq 2)then tit=tdtnam+' '+iseq1+' FCS PIXEL 2'
if(ipix eq 3)then tit=tdtnam+' '+iseq1+' FCS PIXEL 3'
if(ipix eq 4)then tit=tdtnam+' '+iseq1+' FCS PIXEL 4'
if(ipix eq 5)then tit=tdtnam+' '+iseq1+' FCS PIXEL 5'
if(ipix eq 6)then tit=tdtnam+' '+iseq1+' FCS PIXEL 6'
if(ipix eq 7)then tit=tdtnam+' '+iseq1+' FCS PIXEL 7'
if(ipix eq 8)then tit=tdtnam+' '+iseq1+' FCS PIXEL 8'
iunit=iunit+1
window,iunit,tit=tit,xs=800,ys=400,xpos=470,ypos=570
f1(*)=flag1(ipix,*)
; rjt igood=where(f1 eq 1,nigood)
igood=where(f1 le 1,nigood)
if(nigood le 5)then begin
print,'FINDBETATAUFCSILLUM: not enough good fcs data for iseq, ipix = ',$
iseq1,ipix
print,'... nigood = ',nigood
print,'... aborting'
goto, c999
endif ; nigood le 5
t=t1(igood)
signal=t
signal(*)=fsig1(ipix,igood)
signalin=signal
tin=t
;
if(imedfilter ne 0)then begin
;nmaxpt=7
nmaxpt=11 ; best to have odd value
dummy=signal
avred,dummy,signal,nmaxpt,1,idone ; resample to maximum of 10 points using 
;                                   median. Note media is better than mean
;                                   when the systematic data variation is 
;                                   larger than the noise and there is 
;                                   curvature
dummy=t
avred,dummy,t,nmaxpt,idone ; resample to a maximum of 10 points
endif ; imedfilter ne 0
;
nigood=n_elements(t)
; we can be less conservative here than on the short
; plateaus within an observation where *4.0 is used.
silmax=signal(nigood-1)*1.5
silmin=signal(nigood-1)/1.5
;
plot,tin,signalin,color=10,xsty=1 ; plot un filtered signal
;
rmsval=median(frms1(ipix,igood))
rms=t
rms(*)=rmsval
;
; Note for quick time constants only the first few points will be
; decisive in finding the best fit. Hence in these circumstances
; we should not median filter but use small FCSFRACTION
;
; now condition SIGNAL RMS and T arrays: 
; (i) remove negative points with a warning
; (ii) truncate array to use only data only up to time FRACTION of original
;      FCS duration
;
; (i) remove negative points with a warning
;ineg=where(signal le 0.0,nineg)
ipos=where(signal ge 0.0,nipos)
if(nipos lt nigood)then begin ; there are negative values still
print,'FINDBETATAUFCSILLUM: warning negative FCS signals have been censored'
signal=signal(ipos)
rms=rms(ipos)
t=t(ipos)
nigood=nipos
endif ; nipos ne nigood
;
; (ii) truncate array
if(fraction lt 1.0)then begin
tmax=fraction*max(t1) ; (t1 has been set above to count from zero)
ifract=where(t le tmax,nifract)
if(nifract lt 5)then begin
print,'FINDBETATAUFCSILLUM aborting ...'
print,'... too few points in plateau fraction specified ...'
print,'... imedian, fraction, nifract = ',imedian, fraction, nifract
goto, c999
endif ; nifract lt 5
signal=signal(ifract)
rms=rms(ifract)
t=t(ifract)
nigood=nifract
endif ; fraction lt 1.0
;
; prepare parameters for FINDTAUILLUM/SILLUM1
;
sillumpr=sillumprev(pix) ; (pix) is OK
signalprev=sillumpr
if(silmax le silminlim)then begin
print,'FINDBETATAUFCSILLUM: fcs signal for iseq,ipix',iseq1,ipix
print,'... less than ',silminlim
print,'... aborting'
goto, c999
endif ; silmax le 0.0
;
if(silmin le 0.0 or silmin eq silmax)then begin
silmin=silminlim
print,'FINDBETATAUFCSILLUM: warning neg. or zero min fcs sig. for iseq,ipix',$
iseq1,ipix
print,'... setting SILMIN to ',silmin
endif ; silmin le 0.0 or silmin eq silmax
;
; start cut scheme in beta
; for each trial beta in cut scheme find best fit for fcs illumination
; and tau, calling FINDTAUILLUM.
;
; derived controlling parameters for chop in beta
;
xncut=float(ncut)
top=betamax
bot=betamin
dcut=(top-bot)/(xncut-1.0)
betatrial=bot+dcut*findgen(ncut)
;
if(irmswk eq 0)then fitprecwk=fitprec
if(irmswk eq 1)then begin
fitprecwk=fitprec*dcut/betaprec ;     it is unnecessry to have high precision
if(fitprecwk lt fitprec)then fitprecwk=fitprec
if(fitprecwk gt 0.5)then fitprecwk=0.5 ; fits where the model parameters are
;                                        far from optimum
if(fitprecwk lt 0.1)then fitprecwk=0.1
endif ; irmswk eq 1
;
print,'FINDBETATAUFCSILLUM: bot,top,dcut,betatrial = ',bot,top,dcut,betatrial
;
beta=betatrial(0)
; find sillum,tau combination giving best fit (with chi-squared-like 
; value FITRESULT and fit MODELFIT) for trial beta and fixed SIGNALPREV,
; SILLUMPR VALUES. Note the initial call to find starting value for 
; FITRESULT uses fitprec rather than fitprecwk
findtauillum,beta,tau,signalprev,sillumpr,sillum,t,signal,rms,$
tauprec,fitprec,imodel,silmin,silmax,taumin,taumax,isquare,fitresult,modelfit
print,'FINDBETATAUFCSILLUM: beta,tau,fitresult = ',beta,tau,fitresult
;
icutmin=0
fitresultmin=fitresult
fitresultbegin=fitresult
fitresultout(pix)=fitresultmin ; (pix) is OK!!
tauout(pix)=tau
betaout(pix)=beta
sillumout(pix)=sillum
modelfitout=modelfit ; best model fit (for overplotting)
;
iter=0
;
c100:
;
for icut=0,ncut-1 do begin
;
beta=betatrial(icut)
; find sillum,tau combination giving best fit (with chi-squared-like 
; value FITRESULT and fit MODELFIT) for trial beta and fixed SIGNALPREV,
; SILLUMPR VALUES
findtauillum,beta,tau,signalprev,sillumpr,sillum,t,signal,rms,$
tauprec,fitprecwk,imodel,silmin,silmax,taumin,taumax,isquare,fitresult,modelfit
print,'FINDBETATAUFCSILLUM: beta,tau,fitresult,fitresultmin = ',$
beta,tau,fitresult,fitresultmin
;
if(fitresult lt fitresultmin)then begin ; found new minimum
icutmin=icut
fitresultmin=fitresult
fitresultout(pix)=fitresultmin ; (pix) is OK!!
tauout(pix)=tau
betaout(pix)=beta
sillumout(pix)=sillum
modelfitout=modelfit ; best model fit (for overplotting)
print,'FINDBETATAUFCSILLUM: iter,tauout,betaout,sillumout,fitresultmin = ',$
iter,tauout(pix),betaout(pix),sillumout(pix),fitresultmin
endif ; fitresult lt fitresultmin
;
endfor ; icut
;
if(dcut le betaprec)then goto, c200
d=top-bot
d=d/reduce
d2=d/2.0
bot=betaout(pix)-d2
top=betaout(pix)+d2
bottmp=bot
toptmp=top
if(bot lt betamin)then begin
bottmp=betamin
toptmp=bottmp+d
endif ; bot
if(top gt betamax)then begin
toptmp=betamax
bottmp=toptmp-d
endif ; top
bot=bottmp
top=toptmp
dcut=(top-bot)/(xncut-1.0)
betatrial=bot+dcut*findgen(ncut)
;
if(irmswk eq 0)then fitprecwk=fitprec
if(irmswk eq 1)then begin
fitprecwk=fitprec*dcut/betaprec ;       it is unnecessry to have high precision
if(fitprecwk lt fitprec)then fitprecwk=fitprec
if(fitprecwk gt 0.5)then fitprecwk=0.5 ; fits where the model parameters are
;                                        far from optimum
if(fitprecwk lt 0.1)then fitprecwk=0.1
endif ; irmswk eq 1
;
; always find a new minimum for each value of dcut. Because the fit result is
; decreased not only due to the approach to the correct tau, but also by
; the decrease in FITPRECWK in each iteration:
;if(irmswk eq 1)then fitresultmin=fitresultbegin ; don't do this. It will
;                                ; make convergence happen at false minimum
iter=iter+1
print,'FINDBETATAUFCSILLUM: iter,bot,top,betatrial ',iter,bot,top,betatrial
;
goto, c100
;
c200:
;
;oplot,t,modelfitout,color=50 ; overplot best fit (median filtered and clipped
;                                                  data)
;
; find model fit (SIGNALOUT array) at original time sampling (T1) 
; and take value at last sample of SIGNALOUT for SIGNALENDOUT
;
sillumoutarray=t1
sillumoutarray(*)=sillumout(pix)
if(imodel eq 1)then foukes,betaout(pix),tauout(pix),signalprev,sillumpr,$
sillumoutarray,t1,signalout ; (pix) is OK!!
if(imodel eq 2)then expfun,betaout(pix),tauout(pix),signalprev,sillumpr,$
sillumoutarray,t1,signalout ; (pix) is OK!!
nnnn=n_elements(signalout)
signalendout(pix)=signalout(nnnn-1)
;
oplot,t1,signalout,color=50 ; overplot best fit (sampled at all input time 
;                              intervals
;
endfor ; pix
;
; now output value of fits and asymptote to the FCSCAL common block
;
; create arrays if they don't exist:
if(array eq 'c100')then npix=9
if(array eq 'c200')then npix=4
if(n_elements(foundbefore) eq 0)then foundbefore=lonarr(npix)
if(n_elements(foundafter) eq 0)then foundafter=lonarr(npix)
if(n_elements(sillumbefore) eq 0)then sillumbefore=fltarr(npix)
if(n_elements(sillumafter) eq 0)then sillumafter=fltarr(npix)
if(n_elements(sillumprevbefore) eq 0)then sillumprevbefore=fltarr(npix)
if(n_elements(sillumprevafter) eq 0)then sillumprevafter=fltarr(npix)
if(n_elements(fitendbefore) eq 0)then fitendbefore=fltarr(npix)
if(n_elements(fitendafter) eq 0)then fitendafter=fltarr(npix)
;
if(ibefore eq 1)then begin
sillumprevbefore(pixarray)=sillumprev(*)
sillumbefore(pixarray)=sillumout(*)
foundbefore(pixarray)=1
fitendbefore(pixarray)=signalendout(*)
endif ; ibefore eq 1
;
if(ibefore ne 1)then begin
sillumprevafter(pixarray)=sillumprev(*)
sillumafter(pixarray)=sillumout(*)
foundafter(pixarray)=1
fitendafter(pixarray)=signalendout(*)
endif ; ibefore eq 1
;
print,'FINDBETATAUFCSILLUM results ....'
print,'ipix,sillumprev,signalendout,sillumout,betaout,tauout,fitresultout'
;
for pix=0, npixloop-1 do begin
;
ipix=pixarray(pix)
print,ipix,sillumprev(pix),signalendout(pix),sillumout(pix),$
betaout(pix),tauout(pix),fitresultout(pix)
if(betaout(pix) eq betamin or betaout(pix) eq betamax)then $
print,'warning - beta found is at boundary of search range'
if(tauout(pix) eq taumin or tauout(pix) eq taumax)then $
print,'warning - tau found is at boundary of search range'
test=(sillumprev(pix)-sillumout(pix))/sillumprev(pix)
if(abs(test) lt 0.1)then begin ; flux step may be too small for reliable
                               ; determination of detector parameters
print,'warning - fcs too closely matched to sky ...'
print,'ipix,sillumprev,sillumout = ',ipix,sillumprev(pix),sillumout(pix)
endif
;
endfor ; second pix loop
;
c999:
;
return
end





