pro findillumbeforefcs,ipix,ibefore,bestsky,nbestsky,sillum0,ifound,ibestsky,$
iverbose
;
; finds illumination SILLUM0 seen by detector pixel IPIX with chopper
; at central position either before
; start of raster (case IBEFORE=1) or after raster (case IBEFORE=0) by
; examining driftcorrected brightnesses on a natural P32 grid in BESTSKY.
; SILLUM0 is then taken to be the illumination immediately preceding
; the FCS illumination. If IBESTSKY is 0, the value for SILLUM0
; is taken from the median value of all good samples in the S array from
; a map made in pixel ipix. Note that the output SILLUM0 from BESTSKY has
; no vignetting correction.
; 
;
; input:  
;         ARRAY       (common  STEERIN) = 'c100' or 'c200'
;         IBESTSKY     =1 to take value from BESTSKY =0 to take value
;                      from map of S array in the pixel considered.
;         BESTSKY      (1D array in V/s dimension number of directions on
;                      P32 natural grid). This is the illumination that each 
;                      point on the natural sky grid would receive in the
;                      absence of vignetting. 
;                      BESTSKY is left zero where there are no contributing 
;                      data points. 
;         NBESTSKY     (ID integer array) Total number of elementary 
;                      data samples contributing to the value for BESTSKY.
;         IBEFORE      =1 to find illumination 
;         IPIX         identifier of pixel in array.
;         IVERBOSE     =1 not to supress prints
;         common PLOTTER
;         NYRASTER,NZRASTER = dimensions of s/c raster
;         common MAPPER nygrid,nzgrid = dimensions of P32 natural grid
;                     sgridy,sgridz = sampling of natural grid in Y,Z in arcsec
;         common STEER1 npix,pixoff(npix,npix)
;
; output:
;         SILLUM0     (single number in V/s): illumination
;                     prior to the first data point, set to value of BESTSKY 
;                     at the end of the first chopper sweep.
;         IFOUND      = 1 if new values for SIGNAL0 SILLUM0 could be found
;                     = 0 otherwise. 
;
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
@common_steer1
@common_mapper
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_plotter
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
;
ifound=0
;
if(array eq 'c100')then begin
nchop=13 
pixoffset=pixoff/max(pixoff)
pixoffset=long(2.0*(pixoffset-min(pixoffset))/(max(pixoffset)-min(pixoffset)))
pixoffset=pixoffset*3
; = 0 or 3 or 6 (ie detector pixel positions in units of 1/3 detector pixels,
; with minimum position 0 in Y and Z
endif ; array eq 'c100'
;
if(array eq 'c200')then begin
nchop=7 
pixoffset=pixoff/max(pixoff)
pixoffset=long(3.0*(pixoffset-min(pixoffset))/(max(pixoffset)-min(pixoffset)))
; = 0 or 3 (ie detector pixel positions in units of 1/3 detector pixels,
; with minimum position 0 in Y and Z
endif ; array eq 'c200'
;
; find iy1 iz1 - the position on P32 natural grid of detector pixel IPIX
; at start of map when chopper is at central position. Note that ISO rasters
; start at low Y and high Z
;
iy1=nchop/2+pixoffset(0,ipix) ; position of detector pixel in units of Y
                              ; map pixels from map pixel iy=0
workz=round(pixoffset*sgridy/sgridz) ; units of Z map pixels starting at 
;                                     0 eg 0,2,4 for C100 at o/s 2/3
workz=workz-max(workz)+nzgrid-1  
iz1=workz(1,ipix)              ; position of detector pixel in units of Z
                              ; map pixels from map pixel iz=0
;
; find iy2 iz2 - the position on P32 natural grid of detector pixel IPIX
; at end of map when chopper is at central position. Note that ISO rasters
; start at low Y and high Z
;
evenodd,nzraster,iodd
if(iodd eq 0)then iy2=iy1 ; for even rasters the raster ends at same Y position
                          ; as it started
if(iodd eq 1)then begin
worky=pixoffset-max(pixoffset)+nygrid-1-nchop/2 ; max value ngridy-1-nchop/2
iy2=worky(0,ipix)
endif ; iodd eq 1
;
workz=workz-min(workz)
iz2=workz(1,ipix)
;
iy=iy2
iz=iz2
if(ibefore eq 1)then begin
iy=iy1
iz=iz1
endif ; ibefore eq 1
;
; find k identifier 1D key for P32 natural grid as used by BESTSKY
;
if(ibestsky eq 1)then begin ; use BESTSKY array of corrected photometry
; (NB this has no vignetting!!)
k=iy+iz*nygrid
;
; find brightness from best sky
;
sillum0=bestsky(k)
sillum0=double(sillum0)
if(sillum0 gt 0.0)then $
if(iverbose eq 1)then print,'FINDILLUMBEFOREFCS: ipix,ibefore,iy,iz,k = ',$
byte(ipix),byte(ibefore),byte(iy),byte(iz),k
if(sillum0 le 0.0)then begin
print,'FINDILLUMBEFOREFCS: aborting - no data in BESTSKY for ipix,k = ',ipix,k
goto, c999
endif ; sillum0 le 0.0
;
endif ; ibestsky eq 1
;
if(ibestsky ne 1)then begin ; find from S array via MAKEMAP
;
makemap,1,[ipix],mapb,mapr,mapd,mapy,mapz,mapn,0,1
if(mapn(iy,iz) gt 0)then begin
sillum0=mapb(iy,iz)
if(iverbose eq 1)then $
print,'FINDILLUMBEFOREFCS: ipix,ibefore,iy,iz,sillum0 = ',$
byte(ipix),byte(ibefore),byte(iy),byte(iz),sillum0
endif ; mapn(iy,iz) gt 0
if(mapn(iy,iz) le 0)then begin
if(iverbose eq 1)then $
print,'FINDILLUMBEFOREFCS: aborting - no data from MAKEMAP for ipix,iy,iz = ',$
byte(ipix),byte(iy),byte(iz)
goto, c999
endif ; mapn(iy,iz) le 0
;
endif ; ibestsky ne 1
;
ifound=1
;
c999:
;
return
end




