pro findp32grid,ny,nz
;
; finds natural grid of map for data with otf set for P32 aot. This grid is
; the one which minimises resampling necessary to make a uniformly sampled map.
; We make  fixed parameters for the gridding (not dependent on minor statistical
; fluctuations from one observation to the next. For maps drz (routine init1)
; should be tuned so that good registration is obtained between natural sampling
; and the grid defined by FIND32GRID even for big maps. If this is not 
; possible drz should be redefined by examining the difference in pointings 
; between last and first raster points in z.
; Similarly, in Y the difference should be checked to see if the dyc chopper
; increment is really giving good registration even at large Y in a big NY
; raster. If notthe raster dimensions should be defined by dry and not dcy.
;
; MAR99 SGRIDY and SGRIDZ defined by distance between fine pointing centres
; dry and DRZ (set by POINT derived from IIPH - is not the nominal values
;              set by INIT1)
; divided by the array and o/s - dependent number of samples between the
; find pointings. This ensures no loss or registration for very big maps.
;
; For scans sgridz is put equal to the 
;
; Note that for some modes (eg os = 0.66 or os=3 with nz=1 some columns have 
; little or no data)
;
;      input:
; OSAMP   the oversampling factor (or repetition factor if negative) (real)
;         (0.5, 0.66, 1.0, 2.0 or 3.0 for maps -1.0 or -2.0 for scans)
; ARRAY   either 'C100' or 'C200'
; NY NZ   dimensions of s/c raster
; DCY     chopper increment in arcsec
; dry DRZ spacecraft raster increments in arcsec
; NCHOP   number of chopper pointions per sweep.
;
;      output:
; SGRIDY SGRIDZ spacing of grid points in arcsec
; NYGRID NZGRID number of grid points in natural grid 
; YGRID ZGRID   positions of grid points in arcsec wrt map centre in s/c 
;               coordinates
;
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
;
@common_steerin
@common_steer1
@common_mapper
;
if(array eq 'c200')then begin
sgridz=drz
if(osamp eq 3.0)then begin
sgridz=drz
nzgrid=nz-1+4
endif
if(osamp eq 2.0)then begin
sgridz=drz
nzgrid=nz-1+3
endif
if(osamp eq 1.0)then begin
sgridz=drz
nzgrid=nz-1+2
endif
if(osamp lt 0.0)then begin ; scan
sgridz=pixpitch
nzgrid=2
endif
;
endif ; c200
;
if(array eq 'c100')then begin
if(osamp eq 3.0)then begin
sgridz=drz
nzgrid=nz-1+7
endif
if(osamp eq 2.0)then begin
sgridz=drz
nzgrid=nz-1+5
endif
if(osamp lt 0.0)then begin ; scan
sgridz=pixpitch
nzgrid=3
endif
if(osamp eq 1.0)then begin ; scan and map
sgridz=drz
nzgrid=nz-1+3
endif
if(osamp eq 0.5)then begin
sgridz=drz/2.0
nzgrid=((nz-1)*2)+3
endif
if(osamp eq 0.66)then begin
sgridz=drz/3.0
nzgrid=((nz-1)*3)+5
endif
;
endif ; c100
;
;sgridy=dcy
;if(ny gt 1)then sgridy=dry/round(dry/dcy) 
; to get good registration at map edge
gdiv=3.0D ; for c200 array
if(array eq 'c100' and osamp ge 1.0)then gdiv=4.0D ; step 60 arcsec nominal 
if(array eq 'c100' and osamp lt 1.0)then gdiv=6.0D ; step 90 arcsec nominal 
; ***
if(array eq 'c100' and osamp lt 0.0)then gdiv=4.0D ; step 60 arcsec nominal 
sgridy=dry/gdiv ; since dry is true value set by POINT, there will be
                ; no registration problems from large maps
if(array eq 'c100')then nygrid=((ny-1)*round(dry/sgridy))+nchop+6 ; c100 
if(array eq 'c200')then nygrid=((ny-1)*round(dry/sgridy))+nchop+3 ; c200 
;
; now set up YGRID ZGRID
;
ygrid=fltarr(nygrid)
zgrid=fltarr(nzgrid)
ygrid=(findgen(nygrid)*sgridy)-(sgridy*float(nygrid-1)/2.0)
zgrid=(findgen(nzgrid)*sgridz)-(sgridz*float(nzgrid-1)/2.0)
;
print,'FINDP32GRID: sgridy,sgridz,nygrid,nzgrid: ',sgridy,sgridz,nygrid,nzgrid
;print,'FINDP32GRID: ygrid:',ygrid
;print,'FINDP32GRID: zgrid:',zgrid
;
return
end
