pro findrms,badprev,nrelmin
;
; find rms from s (ie signal after relativisation) for each chopper plateau 
; and each pixel. Ignore points denoted bad by badprev=1 and ignore chopper 
; plateaus where there are less than nrelmin good signal points.
; To avoid bias due to drifts and glitches before examining statistics do a 
; polynomial fit to median filtered good signals on 
; each plateau (from all ndrs) and find residue from fit.
;
; Statistics are a conversion of quartiles to rms
;
; make sure that polynomial fits with number of degrees of freedom
; close to that of remaining good points (not flagged bad by glitches etc)
; do not happen, as this would give very low or zero errors.
;
; notes: in finding rms on each ramp we doe not pair differentiate signals
; as this wouldd propagate glitchs. The object of the routine is to
; find an rms from the internal consistency of the data on plateaus, 
; then filtered from the noise so as to give a statistically precise and 
; consistent target for the next deglitch iteration. Note that we do
; not smooth the rms found over ramps since there may be real variations
; in rms between ramps due to signal illumination (shot noise) and 
; drifts in responsivity. After the illumination has been iterated and the
; responsivity vs time found once could attempt to improve precision by fitting
; rms versus const1 + const2*sqrt(illum)
;
; we do not attempt to find noise for individual readout classes as statistics
; are poor. however the rms for each reaout class is scaled by the scaling
; factors from relativise.
;
; another refinement which will be needed will be to do a larger degree
; of freedom fit for points taken during slews.
;
; input:
;         s(npix,nptin) Vs-1 (=V(i+1)-V(i)) relativised
;         rs(nptin) readout cycle (eg  1 2 3 4 1 .... for ninramp=4)
;         nrelmin ; minimum degrees of freedom to find fit to plateau 
;         nch there are nch+2 chopper transitions
;         for the chopper plateau to be included in the relativisation and rms
;         calculation
;    KF=lonarr(nch+2,nkfmax) denotes identifiers within all the chopper plateaus
;         irrespective of readout class
;         NKF(nch+2) is number of reads on a given chopper plateau
;         NKFMAX=maximum of NKF
;         TIME time in sec (DOUBLE dim nptin)
;
; output: (all common block)
;         rms(npix,nptin) estimated rms of each point
;
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;common timer,time
;
@common_steerin
@common_steer1
@common_plotter
@common_signal
@common_timer
;
;rms=dblarr(npix,nptin) ; already initialised in dg1
sigprms=dblarr(npix,nch+2) ; rms for each plateau from all data 
sigillum=dblarr(npix,nch+2) ; illumination for each plateau
badplat=lonarr(npix,nch+2) ; = 1 where there are not enough total points on
;                                a plateau to fit a polynomial
rfit=dblarr(npix,nch+2,npolyrms+1) ; fit coefficients to plateau data 
for ipix=0L,npix-1 do begin             ; data
print,'FINDRMS: computing statistics for pixel',ipix+1
for ichop=-1L,nch do begin ; -1 to nch range is to include first and last too
;
; find fits to each ramp from median filtered good signals
;
;    KF=lonarr(nch+2,nkfmax) denotes identifiers within all the chopper plateaus
;         irrespective of readout class
;         NKF(nch+2) is number of reads on a given chopper plateau
;         NKFMAX=maximum of NKF
i1=kf(ichop+1,0)
i2=i1+nkf(ichop+1)-1
nnn=i2-i1+1
x=dblarr(nnn)
t=x
b=lonarr(nnn)
x=s(ipix,i1:i2)
t=time(i1:i2)
b=badprev(ipix,i1:i2)
igood=where(b eq 0,ngood)
;
if(ngood gt 0)then begin ; rjt oct99
xgood=dblarr(ngood)
tgood=xgood
xgood=x(igood) 
tgood=t(igood)
tgood=tgood-tgood(0)
sigillum(ipix,ichop+1)=median(xgood) ; illumination for each plateau
endif ; ngood gt 0
;
if(ngood ge npolyrms+3)then begin
xx=median(xgood,nrmsmed)
ttest=poly_fit(tgood,xx,npolyrms)
;print,ttest
;help,ttest
rfit(ipix,ichop+1,0:npolyrms)=ttest(0:npolyrms) ; eg 3 terms for npolyrms=2
; now find rms from residuals of all data on ramp 
y=rfit(ipix,ichop+1,0)+(rfit(ipix,ichop+1,1)*tgood)+$
(rfit(ipix,ichop+1,2)*tgood*tgood)
res=xgood-y
qfind,res,ngood,xq1,xq2,xq3
factor=float(ngood)/float(ngood-npolyrms)
sigprms(ipix,ichop+1)=(xq3-xq1)*0.741*sqrt(factor)  ; rms for each plateau from all data 
;         factors to create sigma from quartile difference for normal
;         distribution and to take into account lowering of degrees of
;         freedom from polynomial fit.
;
;if(ichop+1 ge nch-1)then $ diagostic print
;print,'FINDRMS: i1, i2, ipix, ichop+1, sigprms(ipix,ichop+1):', i1,i2,$
;ipix, ichop+1, sigprms(ipix,ichop+1)
;
endif ; npolyrms + 3
;
if(ngood lt npolyrms+3)then badplat(ipix,ichop+1)=1L 
;
endfor ; ichop
;
for ichop=-1L,nch do begin ; -1 to nch range is to include first and last too
;
; interpolate rms for plateaus where not enough good data found. Use
; neigboring points if a good rms found there, otherwise take median of
; all rmses for the pixel in question.
;
if(badplat(ipix,ichop+1) gt 0 or sigprms(ipix,ichop+1) le 0.0)then begin
i1=ichop
i2=ichop+2
if(i1 lt 0)then begin
i1=ichop+2
endif
if(i2 gt nch+1)then begin
i2=i1
endif
xrms1=sigprms(ipix,i1)
xrms2=sigprms(ipix,i2)
if(badplat(ipix,i1) gt 0 or sigprms(ipix,i1) le 0.0)then $
xrms1=median(badplat(ipix,*))
if(badplat(ipix,i2) gt 0 or sigprms(ipix,i2) le 0.0 )then $
xrms2=median(badplat(ipix,*))
sigprms(ipix,ichop+1)=(xrms1+xrms2)/2.0
endif ; ichop
endfor ; ichop
;
; Median filter the rms for plateaus (fluctuations in rms are not real -
; just statistical and could mean good data being thrown away. In any case
; the noise is read and glitch dominated). First plateaus and last plateaus
; set to median of whole signal - these values must be set consevatively
; as they may be the only measurement of a point at the end of a scan.
;
; In addition to median filter convolve (to prevent unphysical plateau to 
; plateau variations).
;
if(nchopmed gt nch+2)then begin
nchopmed=nch+2
print,'FINDRMS: resetting nchopmed to ',nchopmed
endif
xmednoise=median(sigprms(ipix,0:nch+1))
print,'median noise in pixel is: ',xmednoise
;print,'sigprms(ipix,0:10): ',sigprms(ipix,0:10)
dummy=dblarr(nch+2)
dummy(*)=sigprms(ipix,*)
dummy=median(dummy,nchopmed)
c=dblarr(nchopmed)
c(*)=1.0/float(nchopmed)
dummy=convol(dummy,c)
sigprms(ipix,0:nch+1)=dummy
sigprms(ipix,0:nchopmed/2)=sigprms(ipix,nchopmed/2)
sigprms(ipix,(nch+1-nchopmed/2):nch+1)=sigprms(ipix,nch+1-nchopmed/2)
;
for ichop=-1L,nch do begin ; -1 to nch range is to include first and last too
;
; set up rms array (only for points with data)
;
ii1=kf(ichop+1,0)
ii2=ii1+nkf(ichop+1)-1
;if(ipix eq 0 and ichop eq -1)then begin
;print,ii1,ii2
;endif
rms(ipix,ii1:ii2)=sigprms(ipix,ichop+1) 
;if(ichop+1 ge nch)then $ diagostic print
;print,'FINDRMS: ii1,ii2,ipix,ichop+1,sigprms(ipix,ichop+1),rms(ipix,13304):',$
;ii1,ii2,ipix, ichop+1, sigprms(ipix,ichop+1), rms(ipix,13304)
endfor ; ichop
;
;print,'FINDRMS: total(badplat) for pixel = ',total(badplat(ipix,*))
;
;correct rms for factors in relativise
;
for ird=0L,ninramp-2 do begin
iii=where(rs eq ird+1)
;if(ipix eq 0)then help,iii
rms(ipix,iii)=rms(ipix,iii)/rel(ipix,1,ird)
;if(iii eq 13304)then print,'FINDRMS: ipix,iii,rms(ipix,iii) = ',$
;ipix,iii,rms(ipix,iii)
endfor ; ird
;print,'FINDRMS: ipix,rms(ipix,13304) = ',ipix,rms(ipix,13304) ; diagnostic
endfor ; ipix
;
; investigate dependence of rms sigrms on illumination sigillum
;
;window,12,xs=400,ys=400,tit='rms versus illumination pixel 6'
;print,'FINDRMS: plotting pixel 5:'
;plot,sigillum(5,*),sigprms(5,*),psym=1
;window,11,xs=400,ys=400,tit='rms read 2 versus rms read 1'
;plot,sigrms(5,*,0),sigrms(5,*,1),psym=1
;window,12,xs=400,ys=400,tit='rms 2nd read versus readout'
;plot,sigrms(5,*,1)
;
; diagnostic check for zero noise
;
for ipix =0L, npix-1 do begin
for i=0L, nptin-1 do begin
if (rms(ipix,i) le 0.0 and rdstat(i) eq 0 and i eq 13304)then print,$
'FINDRMS warning: ipix,i, rms(ipix,i) = ',ipix,i,rms(ipix,i)
endfor
endfor
;
return
end
