pro findstart,istartnoexpand,backminvthispix,ipix,imodel,icalmethod,$
sig0,sigshoot0,sill0
;
; routine for finding previous signal and illumination prior to
; time range of signals to be analysed. One element of the
; output arrays sig0(npix) sigshoot0(npix) and sill0(npix) are overwritten:
; these arrays must have been dimensioned in the calling program. The
; calculation is model dependent. Also, since for imodel=3 the starting
; state depends on beta, the calculation is dependent on ICALMETHOD.
;

print,'FINDSTART: istartnoexpand,backminvthispix,ipix,imodel,icalmethod = '
print,'... ',istartnoexpand,backminvthispix,ipix,imodel,icalmethod
;
findstartdetnew,istartnoexpand,ipix,siginit,sillinit,sigstart
;
sill0(ipix)=sillinit     ; valid for all models
;
; check sill0 greater than the small positive amount BACKMINVTHISPIX 
;
if(sig0(ipix) lt backminvthispix)then begin
dum=sig0(ipix)
sig0(ipix)=backminvthispix
print,'FINDSTART: warning - sig0 out of bounds for pixel ',ipix
print,' ... resetting sig0 from ',dum,' to ',sig0(ipix)
endif ; sig0(ipix) lt backminvthispix
;
if(sill0(ipix) lt backminvthispix)then begin
dum=sill0(ipix)
;sill0(ipix)=backminthispix
sill0(ipix)=backminvthispix      ;replaced above line, irk, 09/2001
print,'FINDSTART: warning - sill0 out of bounds for pixel ',ipix
print,' ... resetting sill0 from ',dum,' to ',sill0(ipix)
endif ; sill0(ipix) lt backminvthispix
;
if(imodel eq 1 or imodel eq 2)then begin
sig0(ipix)=siginit 
sigshoot0(ipix)=0.0 
endif ; imodel eq 1 or imodel eq 2
;
if(imodel eq 3)then begin
; 
; the following is for the alternative empirical model. The shooting
; and non-shooting signal prior to the first data point is split according
; to the (flux-dependent) betashoot, where the flux dependence is calculated
; from the illumination prior to the first plateau. Hence SETPARCAL and
; SETSHOOTBETA must be called.
;
setparcal,icalmethod,imodel,ipix,betawk,tauwk
sillin=sill0(ipix)
setshootbeta_model3,betawk,sillin,betalong,betashort
sigshoot0(ipix)=betashort*sillin
sig0(ipix)=(1.0-betashort)*sillin
print,'DRIFT32: ipix,sill0,sig0,sigshoot0 = ',ipix,sill0(ipix),sig0(ipix),$
sigshoot0(ipix)
;
; boost sigshoot0 according to the excess of starting signal over 
; illumination from previous plateau.
;
;if(sigstart ne 0.0)then begin ; a valid signal could be found for first
;                                chopper plateau
;sigshoot0(ipix)=sigshoot0(ipix)+betashort*(sigstart-sill0(ipix))
;endif ;  sigstart ne 0.0
;
endif ; imodel eq 3
;
; check sig0 greater than the small positive amount BACKMINVTHISPIX 
;
if(sig0(ipix) lt backminvthispix)then begin
dum=sig0(ipix)
sig0(ipix)=backminvthispix
print,'FINDSTART: warning - sig0 out of bounds for pixel ',ipix
print,' ... resetting sig0 from ',dum,' to ',sig0(ipix)
endif ; sig0(ipix) lt backminvthispix
;
sig0=float(sig0)
sigshoot0=float(sigshoot0)
sill0=float(sill0)
;
return
end





