pro findstartendplateau,from,to,ipointstart,ipointstop,$
iplatstart,iplatstop,istartexpand,istopexpand,istartnoexpand,istopnoexpand
;
;save,/xdr,from,to,ipointstart,ipointstop,$
;file='diagnostic_findstartendplateau.xdr'
;
; From an existing plateau structure of the extended (ie after running 
; EXPAND32) dataset, 
; FINDSTARTENDPLATEAU finds the starting and ending plateaus IPLATSTART and
; IPLATSTOP corresponding to start and finish of s/c fine pointing 
; IPOINTSTART and IPOINTSTOP, such that IPLATSTART and
; IPLATSTOP are the first and last plateaus 
; of the fine pointings IPOINTSTART and IPOINTSTOP which have all samples
; with FPOINT=1. This dependence on FPOINT is implicit, achieved via use of the
; arrays OTFSTART and OTFSTOP in common PLOTTER as accessed by the 
; called routine STARTENDPOINT, by assuming that OTFSTART and OTFSTOP embrace
; a complete set of samples all of which have FPOINT=1. In other words, the
; definition of OTFSTART OTFSTOP must be the same as FPOINT. No access is made
; to the FPOINTWK array used internally by DRIFTCORR. 
;
; FINDSTARTENDPLATEAU is needed eg to find a sequence
; of points for which the first plateau is complete with which to compare
; with a subsequent (possibly the last) plateau in order to find
; the initial detector state. 
;
; input:
;                IPOINTSTART First spacecraft fine pointing (in time order)
;                            in sequence 
;                IPOINTSTOP  Last spacecraft fine pointing (in time order)
;                            in sequence
;                   TO (integer array dimension NPLAT)
;                      identifier in original input time sequence of LAST 
;                      read in each pointing plateau, including single sample
;                      plateaus taken on the fly. The counter include data
;                      not transmitted.
;                 FROM (integer array dimension NPLAT)
;                      identifier in original input time sequence of FIRST 
;                      sample in each pointing plateau, including single sample
;                      plateaus taken on the fly. The counter include data
;                      not transmitted.
;                IREDUCE (common e32) array = 0 for transmitted data
;                                           = 1 for not transmitted data
;
; output:
;                IPLATSTART plateau identifier of first plateau will all 
;                       samples having FPOINT=1
;                IPLATSTOP plateau identifier of last plateau will all 
;                       samples having FPOINT=1
;                ISTARTEXPAND  integer sample identifier of first read on 
;                       plateau IPLATSTART in the extended data set
;                ISTOPEXPAND  integer sample identifier of last read on plateau
;                       IPLATSTOP, in the extended dataset or 
;                       N_ELEMENTS(FPOINT), if this is smaller.
;                ISTARTNOEXTEND sample in the unextended data set 
;                       corresponding to ISTARTEXTEND
;                ISTOPNOEXTEND sample in the unextended data set 
;                       corresponding to ISTOPEXTEND
;
@common_e32
;common e32,es,ireduce,eytrue,eztrue,etime,fpoint,echop,estep,ebad,neptin
;
ittt=where(ireduce eq 0,nittt) ; list of transmitted points in expaanded scheme
iexparray=lindgen(n_elements(ireduce)) ; iorigarray contains mapping of
;                                      ; original into extended sampling scheme
iorigarray=iexparray(ittt)          ; it could be generated by expand32 and put
;                                  ; into the common_e32 block to prevent
;                                  ; unnecessary recalculation 
;
startendpoint,ipointstart,ipointstop,$
irasterstart,irasterstop,istartnoexpand,istopnoexpand,$
irasterstarte,irasterstope,istartexpand,istopexpand
;
istart1=istartexpand
istop1=istopexpand
;
; find start of fine pointing sequence in expanded sampling
;
findramp,from,to,istart1,istart2,iplatstart2
; istart2 is starting read of plateau iplatstart2, the plateau 
; containing istart1
if(istart1 ne istart2)then begin ; case istart1 not at start of plateau
iplatstart=iplatstart2+1
istart=from(iplatstart) 
endif ; istart1 ne istart2
if(istart1 eq istart2)then begin ; case istart1 at start of plateau
iplatstart=iplatstart2
istart=from(iplatstart) 
endif ; istart1 eq istart2
istartexpand=istart
;
; find start of fine pointing sequence in original sampling
;
itest=where(iorigarray eq istartexpand,nteststart)
istartnoexpand=itest(0)
if(nteststart ne 1)then begin
print,'FINDSTARTENDPLATEAU: error in determining ISTARTNOEXPAND - aborting'
goto, c999
endif ; nteststart ne 1
;
; find end of fine pointing sequence in expanded sampling
;
findramp,from,to,istop1+1,istop2,iplatstop2
; istop2 is starting read of plateau iplatstop2, the plateau 
; containing istop1+1
if(istop2 eq istop1+1)then begin ; case istop1 at end of a plateau
iplatstop=iplatstop2-1 ; iplatstop=iplatstop2-1 is correct
istop=to(iplatstop)
endif ; istop2 eq istop1+1
if(istop2 ne istop1+1)then begin ; case istop1 not at end of plateau
iplatstop=iplatstop2-1 ; iplatstop=iplatstop2-1 is correct
istop=to(iplatstop)
endif ; istop2 ne istop1+1
istopexpand=istop
;
; find end of fine pointing sequence in original sampling
;
; the following does not work as the last point in a plateau may not
; be transmitted (unlike the first, which is).
;itest=where(iorigarray eq istopexpand,nteststop)
;istopnoexpand=itest(0)
;if(nteststop ne 1)then begin
;print,'FINDSTARTENDPLATEAU: error in determining ISTOPNOEXPAND - aborting'
;goto, c999
;endif ; nteststop ne 1
;
; ....hence 
nplat=n_elements(from)
if(iplatstop eq nplat-1)then istopnoexpand=n_elements(iorigarray)-1
; last expanded point is last point of dataset
;
if(iplatstop ne nplat-1)then begin
itest=where(iorigarray eq istopexpand+1,nteststop) ; first transmitted point
;                                                  of next plateau
if(nteststop ne 1)then begin
print,'FINDSTARTENDPLATEAU: error in determining ISTOPNOEXPAND - aborting'
goto,c999
endif ; nteststop ne 1
;
istopnoexpand=itest(0)-1
;
endif ; iplatstop ne nplat-1
;
c999:
;
return
end









