;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GET_RLIN
;
; PURPOSE:
;	Get signals linearized 
;
; CATEGORY:
;	PIA - Processing
;                
; CALLING SEQUENCE:
;	GET_RLIN, TMP, SIG, SIG_COR 
;
; INPUT PARAMETERS:
;	tmp:		temporary PIA structure (NO update)
;	SIG:		the signals to be linearized 
;	
;
; OUTPUT:
;	sig_cor:	the corrected signals after linearization
;			  or "UNcorrected" if REVERSE keyword set
;		 	
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel - ESA-VILSPA/SAI 		Nov.99
;-
PRO GET_rlin, tmp, sig, sig_cor, REVERSE=reverse

@phtfilter
	
	  newhdr = tmp.hdr

	INFO = ''
	
	sig_cor = sig
	

; directory containing the tables
	tabdir = !CALG + 'rlin_tables/'

; init filters if necessary
	IF N_ELEMENTS(c1filters) EQ 0 THEN phtfilter_init
        allcfilters = [c1filters, c2filters,'C_cut']
        allpfilters = [p1filters, p2filters, p3filters]
 
; Read the Compact Status if necessary or get the unit - change -> shape4
	give_cs, tmp, cst, record, cst_conv, ERROR = isnot_cs
	IF isnot_cs THEN BEGIN 
	  csfile = 'PSTA'+strmid(tmp.admn,4,8)+'.FITS'
	  read_cs, csfile
	  give_cs, tmp, cst, record, cst_conv
	ENDIF  

	CASE STRMID(tmp.det, 0,2) OF
	  'P1': BEGIN
	  	filtnr = cst.c3ps(record) - 1
		npixel = 1
		filter = allpfilters(filtnr)
;		asc_fil = allpfilters(filtnr) + '_1_sig.txt'
		END		
	  'P2': BEGIN
	  	filtnr = cst.c3ps(record) - 1
		npixel = 1
		filter = allpfilters(filtnr)
;		asc_fil = allpfilters(filtnr) + '_1_sig.txt'
		END		
	  'P3': BEGIN
	  	filtnr = cst.c3ps(record) - 1
		npixel = 1
		filter = allpfilters(filtnr)
;		asc_fil = allpfilters(filtnr) + '_1_sig.txt'
		END		
	  'C1': BEGIN
	  	filtnr = cst.c2ps(record) - 2
		npixel = 9
		filter = allcfilters(filtnr)
;		asc_fil = allcfilters(filtnr) + $
;		       '_'+ STRTRIM(indgen(npixel)+1,2)+'_sig.txt'
		END
	  'C2': BEGIN
	  	filtnr = cst.c2ps(record) - 2
		npixel = 4
		filter = allcfilters(filtnr)
;		asc_fil = allcfilters(filtnr) + $
;		       '_'+ STRTRIM(indgen(npixel)+1,2)+'_sig.txt'
		END
	  ELSE: BEGIN
	  	MESSAGE, /INFO, 'Wrong detector, returning ...'
		RETURN
		END
	ENDCASE
	
	IF KEYWORD_SET(REVERSE) THEN reverse=1 ELSE reverse=0
	
	FOR ipix=0,npixel-1 DO BEGIN

	  sigtmp = rlin_fct(sig(ipix,*), filter, ipix+1, det=tmp.det, $
	  		reverse=reverse)
	  sig_cor(ipix,*) = sigtmp
	ENDFOR
	
	RETURN
	END
		
	
