pro glitchrecover,iplat,nplatback,nplatforward,maskout,$   
skyfound,nskyfound,skyfoundfitres,skyfoundnpt,platskyuse,skyfoundplat,$
nskytry,skytryplat,bestsky,nbestsky,skypass,$
fpointwk,keypoint,vig,nplat,nplateau,from,to,vigp,keyp,finep,pident,$
iplatprevarray,iplatprevprevarray,jjhistfollowarray,igoback
;
; GLITCHRECOVER is called when a solution for the illumination on the current 
; plateau IPLAT is out of bounds. This may be because of an inconsistency 
; between the calculated illumination history leading up to IPLAT and the 
; signal. For example, if the preceding plateau IPLAT-1 has a too high presumed
; illumination, the solution for IPLAT will be too low, below the tolerance 
; range for SOLVEILLUM1, especially for low BETA jump factors. In principle,
; this situation may arise because the deduced pointing or the P32 natural 
; grid is wrong (so that the pointing tolerance circle set in POINT overlaps 
; with an adjacent fine pointing position with a contrasting illumination). 
; More likely, it arises because of a glitch in one of the preceding 
; plateaus. Although this should not occur if the deglitch in the signal
; conditioning prior to a call to DRIFT32 has worked, in practise we may
; very occasionally get such a situation if the tolerance SILMIN,SILMAX
; for the plateau solution in the SOLVEILLUM1 calls is not set conservatively.
; These situations must be recognised and prevented as the glitch can 
; currupt the illumination history for many succeeding plateaus. If this
; happens on the first plateau encountered in a given direction, the whole
; solution thereafter may be corrupted.
;
; In the following it is assumed that glitches are "active" in the illumination
; history - that is, an effective virtual illumination
; can be attributed to a glitch to which the detector responds as it the 
; illumination was real. "Passive" glitches which affect the signal but not the
; illumination history (ie where the signal returns instantaneously back to 
; the signal that would have been found without a glitch) are not catered for
; by GLITCHRECOVER. These are more likely to have been found in the
; deglitch in the signal conditioning in ERDTOMAP. (The only way to treat such
; "passive" glitches would be to find a solution for the glitch plateau and 
; the succeeding plateau, with the solution only constrained by the model-
; illumination rms of the succeeding plateau). The virtual illumination for 
; the sequence of plateaus containing te "active" glitch is estimated by 
; solving for the previous NPLATBACK plateaus (irrespective of whether these 
; are fine pointings or not) as if they were data taken on the fly, traversing 
; sources on the slew. To this end, the input FPOINTWK is set to zero by
; GLITCHRECOVER for all elementary points on the last NPLATBACK plateaus,
; and the arrays nplat,nplateau,from,to,vigp,keyp,finep also updated.
; In addition, for any of the past NPLATBACK plateaus which were fine
; pointings, the value of skyfound,nskyfound,skyfoundfitres,skyfoundnpt,
; bestsky,nbestsky and skypass previously set by DRIFTCORR in those plateaus
; are removed. Finally, all points in the sequence of NPLATBACK plateaus
; are flagged bad in the MASKOUT array so they don't contribute to calls
; to the photometry via MAKEMAP etc.
; On return to DRIFTCORR, the NPLATBACK plateaus are recalculated
; as slew (for which none of the illuminations found are stored into BESTSKY
; etc.). 
;
; If GLITCHRECOVER is not effective with a small value for NPLATBACK,
; it could be because the glitch occurred further back in the signal. This 
; should be obvious to the user from inspection of the output from 
; TIMELINEFIGURE. A suitable value for NPLATBACK can be estimated from 
; TIMELINEFIGURE and DRIFT32 re-run with this value. For pipeline application
; NPLATBACK should be set conservatively large - eg to the number of plateaus
; in a chopper sweep. 
;
; GLITCHRECOVER also flags bad (MASKOUT) points in the NPLATFORWARD
; plateaus after the current plateau, and changes these to 
; a slew status. These points are then ignored
; in the calculation of BESTSKY by FINDBESTSKY (in the absence of 
; any other SKYFOUND points from other plateaus), so as not to include 
; possibly unrecovered illuminations in the derivation of the illumination
; history, which would otherwise propagate the effect of the glitch to good
; plateaus.
;
; First update skyfound,nskyfound,skyfoundfitres,skyfoundnpt,
; bestsky,nbestsky prevously found on any of the preceding NPLATBACK plateaus 
; which were fine pointings, plus the current plateau:
;
ip1=iplat-nplatback
;ip2=iplat-1 
ip2=iplat ; current plateau
i1=from(ip1)
i2=to(ip2)
nfineback=0
igoback=1
;
; first flag bad forward points before recalculating TO and FROM arrays 
;
if2=0
if1=0
if(nplatforward gt 0)then begin ; only do if 1 or more forward plateaus
;                                 since we have already set bad current 
;                                 plateau IPLAT
ipf1=iplat+1 
ipf2=iplat+nplatforward
ifineforward=where(finep(ipf1:ipf2) eq 1,nfineforward) ; nfineforward is 
;                         number of fine pointing plateaus set to slew in 
;                         forward direction
if1=from(ipf1)
if2=to(ipf2)
maskout(if1:if2)=1
fpointwk(if1:if2)=0
endif ; nplatforward gt 0
;
;print,'GLITCHRECOVER - iplat,nplatback,nplatforward,i1,i2,if1,if2 = ',$
;iplat,nplatback,i1,i2,if1,if2
print,'GLITCHRECOVER - iplat = ',iplat,' ...'
print,'... nplatback,i1,i2      = ',nplatback,i1,i2
print,'... nplatforward,if1,if2 = ',nplatforward,if1,if2
;
for ip=ip2-1,ip1,-1 do begin ; this loop does not correct SKYFOUND etc for
;                            current plateau (we count from ip2-1) as 
;                            GLITCHRECOVER is called before values for
;                            SKYFOUNFD etc have been assigned for the
;                            current plateau
if(finep(ip) eq 1)then begin
nfineback=nfineback+1
kp=keyp(ip) ; natural grid 1D pointing identifier
if(platskyuse(ip) gt 0)then begin ; we have used this fine pointing
;                                   plateau for skyfound value previously
nsf=nskyfound(kp)
nsftry=nskytry(kp)
if(skyfoundplat(kp,nsf-1) ne ip)then $ should always be true
print,'GLITCHRECOVER - warning: resetting SKYFOUND for incorrect plateau' 
platskyuse(ip)=0
skyfoundplat(kp,nsf-1)=0
skytryplat(kp,nsftry-1)=0
skyfound(kp,nsf-1)=0.0
skyfoundfitres(kp,nsf-1)=0.0 
skyfoundnpt(kp,nsf-1)=0 ; number of data points contributing to fit
nskyfound(kp)=nskyfound(kp)-1
nskytry(kp)=nskytry(kp)-1
endif ; platskyuse(ip) gt 0
endif ; finep(ip) eq 1
;
endfor ; ip=ip2,ip1,-1
;
if(finep(iplat) eq 1)then nfineback=nfineback+1 ; include current plateau
;
maskout(i1:i2)=1 ; update mask array of preceding and current plateaus
;                  whether or not we update plateaus
;
if(nfineback gt 0)then begin ; fine pointings encountered in NPLATBACK and 
;                               current plateaus
;
; Now, if there are fine pointings in preceding or current plateaus reset 
; SKYPASS (since we will go back and recalculate these in calling program) 
; and update FPOINTWK to slew over all NPLATBACK plateaus
; and the current plateau:
;
skypass(i1:i2)=0.0
fpointwk(i1:i2)=0
;
endif ; nfineback gt 0
;
if(nfineback gt 0 or nfineforward gt 0)then begin
;
; update pointing plateau arrays vig,nplat,nplateau,from,to,vigp,
; keyp,finep,pident from new FPOINTWK array: 
;
findplateau,fpointwk,keypoint,vig,nplat,nplateau,from,to,vigp,$
keyp,finep,pident,0 ; notverbose
;
; update platskyuse, iplatprevarray iplatprevprevarray and 
; jjhistfollowarray arrays for new increased dimensioning
;
dummy=lonarr(nplat)
i=where(platskyuse gt 0,ni)
if(ni gt 0)then dummy(i)=platskyuse(i)
platskyuse=dummy
;
dummy=lonarr(nplat)
dummy(*)=-1
i=where(iplatprevarray ge 0,ni)
if(ni gt 0)then dummy(i)=iplatprevarray(i)
ix1=max([0,iplat-nplatback+1])
ix2=iplat
dummy(ix1:ix2)=-1
iplatprevarray=dummy
;
dummy(*)=-1
i=where(iplatprevprevarray ge 0,ni)
if(ni gt 0)then dummy(i)=iplatprevprevarray(i)
dummy(ix1:ix2)=-1
iplatprevprevarray=dummy
;
dummy(*)=-1
i=where(jjhistfollowarray ge 0,ni)
if(ni gt 0)then dummy(i)=jjhistfollowarray(i)
dummy(ix1:ix2)=-1
jjhistfollowarray=dummy
;
endif ; nfineback gt 0 or nfineforward gt 0
;
if(nfineback eq 0)then begin ; no fine pointings found - glitch on slew
; here we must be careful not to enter a recursive loop in calling program.
; Strictly speaking we should solve for the glitch with solveillum2,
; or alternatively extend the period leading to the next fine pointing
; to give the detector chance to recover but instead we just return with the 
; instruction to carry on and not
; recalculate the illumination history.
; 
igoback=0
;
endif ; nfineback eq 0
;
print,'GLITCHRECOVER - nfineback,points masked back and current = ',$
nfineback,i2-i1+1
print,'GLITCHRECOVER - nfineforward,points masked forward       = ',$
nfineforward,if2-if1+1 
;
c999:
;
return
end


