pro initscp
;
; initialises COMMON SCPBLOCK by creating the plateau structure from the
; (unexpanded) STEP array in common plotter. 
; Note that the plateau structures set up by 
; INITSCP differ from those found by FINDPLATEAU in that the chopper plateaus 
; on the fly are retained in their full length. This preserves the possibility
; of an SCP level deglitch on faint sources for P32 even on the fly.
;
; In addition to initialising all the arrays in COMMON SCPBLOCK
; the following data independent arrays are filled using pointing information
; for spacecraft and individual pixels in common PLOTTER and from FPT
; array in common PINFO
;
; because STARTENDPOINT is called this routine should be called after the 
; E32 common block has been filled, even though the program only operates
; on unexpanded data
;
;     KPT_SCP(npix,nplat_scp) pointing direction on P32 natural grid of 
;                             fine pointing direction. For a slew it is the
;                             most common pointing direction of the good points
;                             on the plateau
;     VIG_SCP(nplat_scp)      vignetting factors for each chopper plateau
;     SPT_SCP(nplat_scp)      s/c pointing identifyer (in time order). For a 
;                             slew it is set to -1
;     FPT_SCP(nplat_scp)      =1 if all good or bad points on the plat are on a
;                             fine pointing, =0 otherwise
;     FROM_SCP(nplat_scp)     ERD ident of 1st point of plateau (even if bad)
;     TO_SCP(nplat_scp)       ERD ident of last point of plateau (even if bad).
;     TIME_SCP(nplat_scp)     Time of first sample in each plateau 
;                             (for simplicity) 
;  
; Note the data dependent arrays in the SCPBLOCK common should only be
; filled after calculations of S have been completed 
; (eg after driftcorrection); this should be done by a call to ERDTOSCP
; in the actual application (eg SCP level deglitching using DG2).
;
; In addition the array DG2BAD in common DG2BLOCK is initialised
;
@common_pinfo
;common pinfo,fpt
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_plotter
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster,racenout,deccenout
@common_timer
;common timer,time,dtime,complete
@common_scpblock
;common scpblock,s_scp,bad_scp,rms_scp,n_scp,kpt_scp,spt_scp,fpt_scp,$
;from_scp,to_scp,time_scp,vig_scp,nplat_scp
@common_dg2block 
;common dg2block,idg2badvalue1,idg2badvalue2,dg2bad
@common_signal
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;
dg2bad=bad
dg2bad(*,*)=0
;
if(array eq 'c100')then npix=9
if(array eq 'c200')then npix=4
;
; set up chopper plateau structure
;
n=n_elements(step)
test=step(1:n-1)-step(0:n-2)
itest=1+where(abs(test) gt 0,nitest)
nplat_scp=nitest+1
;
; initialise data dependent arrays
;
s_scp=fltarr(npix,nplat_scp)
n_scp=lonarr(npix,nplat_scp)
bad_scp=n_scp
rms_scp=s_scp
vig_scp=s_scp
;
; initialise data independent arrays
;
kpt_scp=n_scp
spt_scp=lonarr(nplat_scp)
fpt_scp=spt_scp
from_scp=spt_scp
to_scp=spt_scp
time_scp=fltarr(nplat_scp)
;
; fill data independent arrays
;
from_scp(1:nplat_scp-1)=itest
from_scp(0)=0
to_scp(0:nplat_scp-2)=itest-1
to_scp(nplat_scp-1)=n-1
;
; find pixel dependent data independent quantities
;
for ipix=0,npix-1 do begin
;
; find vignetting VIG
if(array eq 'c100')then vignecorr32,1,ipix,chop,vig ; vig has dimensions
if(array eq 'c200')then vignecorr32,2,ipix,chop,vig ; ... npix * N
;
findkeypoint,ipix,ytrue,ztrue,keypoint ; find KEYPOINT from pointing info
;
for iplat=0,nplat_scp-1 do begin
;
i1=from_scp(iplat)
i2=to_scp(iplat)
dum=keypoint(i1:i2)
mostcommon,dum,dummostcommon
kpt_scp(ipix,iplat)=dummostcommon ; most common pointing
;
vig_scp(ipix,iplat)=median(vig(i1:i2)) ; should be all the same
;
endfor ;  iplat=0,nplat_scp-1
endfor ;  ipix=0,npix-1
;
; find erd pointers of start and end of fine pointings
; call just on first pointing since still get all pointings in irasterstart
; and irasterstop arrays
startendpoint,0,0,$
irasterstart,irasterstop,istartnoexpand,istopnoexpand,$
irasterstarte,irasterstope,istartexpand,istopexpand 
;
; find pixel independent data independent quantities
;
for iplat=0,nplat_scp-1 do begin
;
i1=from_scp(iplat)
i2=to_scp(iplat)
ninplat=i2-i1+1
;
fpt_scp(iplat)=0
ifine=where(fpt(i1:i2) eq 1,nifine)
if(nifine eq ninplat)then fpt_scp(iplat)=1
;
ii=where(irasterstart le i1,nii) ; note irasterstart is in time order
iraster1=irasterstart(ii(nii-1))
iraster2=irasterstop(ii(nii-1))
spt_scp(iplat)=-1
if(i1 ge iraster1 and i2 le iraster2)then spt_scp(iplat)=nii-1
;
time_scp(iplat)=time(i1)
;
endfor ; iplat=0,nplat_scp-1
;
return
end







