pro mapcontour,igridover,nlev,mcint,ilog,map,maprms,iwindow,psplot
;
; contours map(o:nygrid-1,0:nzgrid-1) over region containing data
; within a frame o:nygrid-1,0:nzgrid-1 on unit iwindow
;
; If psplot is 'ps' a contour plot is written to a file 'mapcontour.ps'
; In this case the routine calls
; set_plot,'ps'   
; device,filename='mapcontour.ps',xs=***,ys=***
; before the contour (where the sizes *** are calculated to give 
; the correct aspect ratio), followed by
; device,/close 
; set_plot,'x' 
;
; if igridover is 1 the map grid is overlaid
; if ilog is 0 NLEV linear contours are plotted from minimum to maximum with 
;     negative contours dashed; value of MCINT ignored
;  if ilog is -1 linear contours are plotted symetrically about zero, if
;     a zero exists, with interval MCINT
; if ilog is 1 log contours (doubling factor) are plotted after shifting 
;    minimum to the value of MCINT (normally mcint should be specified
;    as about 1.5 rms noise in this case).
;
; Perimeter with value of errflag is not plotted
;
; MAPRMS is input to act as a flag for pixels having no data, which are
;    not plotted and which are not included in the calculation of statistics
;
; a maximum of NCMAX=30 contours are plotted otherwise IDL crashes, starting
; at the lowest contour calculated as described above
;
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;common plotter,otfstart,otfstop,rdstat,chop,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
;common pinfo,fpt
;
@common_mapper
@common_steerin
@common_signal
@common_plotter
@common_steer1
@common_pinfo
;
if(n_elements(igridover) le 0)then begin
print,'mapcontour,igridover,nlev,mcint,ilog,map,maprms,iwindow,psplot'
print,'eg: mapcontour,1,10,0.1,0,mapb,mapr,3,'''' ; linear window 3 10 contours + grid'
print,'eg: mapcontour,0,10,0.1,1,mapb,mapr,5,''ps'' ; log window 5 no grid to ps file'
goto,c999
endif 


;
; plotting in correct proportions
;
xsize=600.0 ; s/c Y window size in pixels
xborder=50 ; screen pixels border
;
x1=min(ygrid)-sgridy/2.0
x2=max(ygrid)+sgridy/2.0
y1=min(zgrid)-sgridz/2.0
y2=max(zgrid)+sgridz/2.0
ysize=xsize*(y2-y1)/(x2-x1)
if(psplot ne 'ps')then window,iwindow,xs=xsize,ysize=ysize,title=' '
xr=[x1,x2]
yr=[y1,y2]
fract=1.0-(xborder/xsize) ; fraction of screen occupied by data
px1=x1*fract
px2=x2*fract
py1=y1*fract
py2=y2*fract
yborder=xborder*(y2-y1)/(x2-x1)
px1=xborder
;px2=xsize-xborder
px2=xsize-xborder/10.0
py1=yborder
;py2=ysize-yborder
py2=ysize-yborder/10.0
if(psplot ne 'ps')then begin
plot,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],pos=[px1,py1,px2,py2],psym=0,xstyle=1,$
ystyle=1,/device,xtit='OFFSET Y (arcsec)',ytit='OFFSET Z (arcsec)'
endif
;
if(psplot eq 'ps')then begin
;
set_plot,'ps'
;
;print,'setting line and character widths for ps file ...'
!p.charsize=1.3 ; annotation size for axes
!p.charthick=2.5 ; thickness for annotation 
!p.thick=2.0  ; thickness for plotted points
!x.thick=2.5  ; thickness of x axis
!y.thick=2.5  ; thickness of y axis
;
xsize=13.0
;
x1=min(ygrid)-sgridy/2.0
x2=max(ygrid)+sgridy/2.0
y1=min(zgrid)-sgridz/2.0
y2=max(zgrid)+sgridz/2.0
ysize=xsize*(y2-y1)/(x2-x1)
device,filename='mapcontour.ps',xs=xsize,ys=ysize 
; for ps plots the aspect ratio is taken care of properly in the plot
; call without need for px1 etc
plot,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],psym=0,xstyle=1,ystyle=1,$
xtit='OFFSET Y (arcsec)',ytit='OFFSET Z (arcsec)'
;
endif ; psplot eq 'ps'
;
if(igridover eq 1)then begin ; overlay map grid
oplot,[x1,x2],[y1,y1],linestyle=0
for iz=1,nzgrid do begin
y=y1+float(iz)*sgridz
oplot,[x1,x2],[y,y],linestyle=0
endfor ; iz
oplot,[x1,x1],[y1,y2],linestyle=0
for iy=1L,nygrid do begin
x=x1+float(iy)*sgridy
oplot,[x,x],[y1,y2],linestyle=0
endfor ; iy
endif ; igridover
;
; contour MAP
;
levfind,nlev,mcint,ilog,map,maprms,lev ; find contour levels LEV
;
ncmax=30
ntoplot=n_elements(lev)
nplot=1+(ntoplot-1)/ncmax ; 30 is maximum number of contours which can be
;                            plotted in a single call of CONTOUR
levmult=fltarr(nplot,ncmax*nplot)
eflag=-99999.0
levmult(*,*)=eflag
istop=-1
for i=0,nplot-1 do begin
istart=istop+1
istop=istart+ncmax-1
if(istop gt ntoplot-1)then istop=ntoplot-1 
print,i,istart,istop,ntoplot
levmult(i,istart:istop)=lev(istart:istop)
endfor ; i
;
findgoodarea,maprms,i1,i2,j1,j2 ; clip off edges containing 
;                                 only dummy data from plot
;
print,'MAPCONTOUR: map pixels plotted: ',i1,i2,j1,j2
;print,'MAPCONTOUR: levels plotted are: ',lev
yplot=min(ygrid)+sgridy*float(i1)+findgen(i2-i1+1)*sgridy
zplot=min(zgrid)+sgridz*float(j1)+findgen(j2-j1+1)*sgridz
;
for i=0,nplot-1 do begin
levplot=levmult(i,*)
ii=where(levplot ne eflag,nii)
if(nii gt 0)then levplot=levplot(ii)
print,'MAPCONTOUR: levels plotted are: ',levplot
if(i eq 0)then contour,map(i1:i2,j1:j2),yplot,zplot,$
lev=levplot,C_LINESTYLE = (levplot LT 0.0),/overplot
if(i gt 0)then contour,map(i1:i2,j1:j2),yplot,zplot,$
lev=levplot,C_LINESTYLE = (levplot LT 0.0),/overplot
endfor ; i
;
if(psplot eq 'ps')then begin
device,/close
set_plot,'x'
print,'written contours to file mapcontour.ps';
;print,'re-setting line and character widths for screen plot...'
!p.charsize=1.0 ; annotation size for axes
!p.charthick=1.0 ; thickness for annotation 
!p.thick=1.0  ; thickness for plotted points
!x.thick=1.0  ; thickness of x axis
!y.thick=1.0  ; thickness of y axis
endif ; psplot eq 'ps'
;
c999:
;
return
end




