pro mapsimul32,array,osamp,ny,nz,nmappix,mappix,$
map,maprms,mapdisp,mapoffy,mapoffz,mapn,iweight,iprint
;
; simulates map on a simple grid (not starting from time) for PHT32 with
; oversampling OSAMP array ARRAY ('c200' or 'C100') and raster pointings NY NZ.
;
; This is useful eg to test tme independent flatfielding routine for various 
; oversamplings. MAP is output map, MAPN is the number of samples per map 
; point (assumes one complete sweep per spacecraft pointing) MAPDISP is the 
; rms dispersion on each grid point/sqrt(number of contributing points) MAPRMS 
; is the containing propagated input 1 sigma error. Also
; output are the weighted mean positions of the data contributing to each map 
; pixel in y and z (MAPOFFY MAPOFFZ) in arcsec.
;
; All arrays are calculated on a natural grid. The map is calculated
; using a Gaussian source (future option beam) and a flat and a dark (all
; time independent). The map is caalculated from NMAPPIX detector pixels
; specified by MAPPIX (IDL notation). If IWEIGHT is 0 then each input data 
; point is given equal weight, and the noise values are ignored.
;
; the routine has no common blocks to avoid overwriting real data. Code has
; been copied from routines INIT1 and FINDP32GRID
;
; unless IPRINT is 1 then the print statements are suppressed
;
back=1.0 ; background
peak=10.0 ; peak brightness of gaussian
if(array eq 'c100')then begin
npix=9
ay=0.0013103 ; exp(- ay * y*y)for 46 arcsec FWHM
az=0.0013103 ; 46 arcsec FWHM
endif
if(array eq 'c200')then begin
npix=4
ay=0.000327574 ; exp(- ay * y*y)for 92 arcsec FWHM
az=0.000327574 ; 92 arcsec FWHM
endif
;
darksimul=findgen(npix)
;darksimul(*)=0.1*findgen(npix)
darksimul(*)=0.0
flatsimul=darksimul
flatsimul(*)=1.0
;flatsimul(2)=2.0
;flatadd=0.2*findgen(npix)
;flatadd=flatadd-total(flatadd)/float(npix)
;flatsimul=flatsimul+flatadd
flatsimul=float(npix)*(flatsimul/total(flatsimul))
rmssimul=darksimul
rmssimul(*)=0.001
;rmssimul(*)=0.0
seed=10
;
   if(array eq 'c100')then begin
   npix=9
   nchop=13
   dcy=double(15.0) ; nominal chop step in Y
   pixsize=double(43.5)
   pixpitch=double(45.9)
   pixoff=dblarr(2,npix)
; chopper offset versus index
  choff=fltarr(nchop)
  choff1=[-6,-5,-4,-3,-2,-1,0,1,2,3,4,5,6]
   choff=float(choff1) * dcy
; offset in Y wrt central pixel
   pixoff(0,*)=[-1.0,-1.0,-1.0,0.0,0.0,0.0,1.0,1.0,1.0]*pixpitch
;   pixoff(0,*)=[-1.0,0.0,1.0,-1.0,0.0,1.0,-1.0,0.0,1.0]*pixpitch
; offset in Z wrt central pixel
   pixoff(1,*)=[-1.0,0.0,1.0,-1.0,0.0,1.0,-1.0,0.0,1.0]*pixpitch
;print,'c100 pixoff(0,*):',pixoff(0,*)
;print,'c100 pixoff(1,*):',pixoff(1,*)
   if(abs(osamp) eq 3.) then begin
   dry=60.0 ; spacecraft raster step in Y
   drz=15.0 ; spacecraft raster step in Z
   endif ; osamp
   if(abs(osamp) eq 2.) then begin
   dry=60.0 ; spacecraft raster step in Y
   drz=23.0 ; spacecraft raster step in Z
   endif ; osamp
   if(abs(osamp) eq 1.) then begin
   dry=60.0 ; spacecraft raster step in Y
   drz=46.0 ; spacecraft raster step in Z
   endif ; osamp
if(abs(osamp) eq 0.66) then begin
   dry=92.0 ; spacecraft raster step in Y
   drz=69.0 ; spacecraft raster step in Z
   endif ; osamp
   if(abs(osamp) eq 0.5) then begin
   dry=92.0 ; spacecraft raster step in Y
   drz=92.0 ; spacecraft raster step in Z
   endif ; osamp
   endif ;'c100'
   if(array eq 'c200')then begin
   npix=4
   nchop=7
   dcy=double(31.0) ; nominal chop step in Y
   pixsize=double(89.4)
   pixpitch=double(91.7)
   pixoff=dblarr(2,npix)
;  chopper offset versus index
   choff=fltarr(nchop)
   choff1=[-3,-2,-1,0,1,2,3]
   choff=float(choff1) * dcy
; offset in Y wrt central pixel
;   pixoff(0,*)=[-0.5,0.5,-0.5,0.5]*pixpitch
   pixoff(0,*)=[0.5,-0.5,-0.5,0.5]*pixpitch
; offset in Z wrt central pixel
;   pixoff(1,*)=[-0.5,0.5,-0.5,0.5]*pixpitch
   pixoff(1,*)=[-0.5,-0.5,0.5,0.5]*pixpitch
   if(abs(osamp) eq 3.) then begin
   dry=92.0 ; spacecraft raster step in Y
   drz=31.0 ; spacecraft raster step in Z
   endif ; osamp
   if(abs(osamp) eq 2.) then begin
   dry=92.0 ; spacecraft raster step in Y
   drz=46.0 ; spacecraft raster step in Z
   endif ; osamp
   if(abs(osamp) eq 1.) then begin
   dry=92.0 ; spacecraft raster step in Y
   drz=92.0 ; spacecraft raster step in Z
   endif ; osamp
   endif ;'c200'
;
if(array eq 'c200')then begin
sgridz=drz
if(osamp eq 3.0)then begin
sgridz=drz
nzgrid=nz-1+4
endif
if(osamp eq 2.0)then begin
sgridz=drz
nzgrid=nz-1+3
endif
if(osamp eq 1.0)then begin
sgridz=drz
nzgrid=nz-1+2
endif
if(osamp lt 0.0)then begin ; scan
sgridz=pixpitch
nzgrid=2
endif
;
endif ; c200
;
if(array eq 'c100')then begin
if(osamp eq 3.0)then begin
sgridz=drz
nzgrid=nz-1+7
endif
if(osamp eq 2.0)then begin
sgridz=drz
nzgrid=nz-1+5
endif
if(osamp lt 0.0)then begin ; scan
sgridz=pixpitch
nzgrid=3
endif
if(osamp eq 1.0)then begin ; scan and map
sgridz=drz
nzgrid=nz-1+3
endif
if(osamp eq 0.5)then begin
sgridz=drz/2.0
nzgrid=((nz-1)*2)+3
endif                         
if(osamp eq 0.66)then begin
sgridz=drz/3.0
nzgrid=((nz-1)*3)+5
endif
;
endif ; c100
;
; calculate grid
;
sgridy=dcy
if(ny gt 1)then sgridy=dry/round(dry/dcy) ; to get good registration at map edge
if(array eq 'c100')then nygrid=((ny-1)*round(dry/sgridy))+nchop+6 ; c100
if(array eq 'c200')then nygrid=((ny-1)*round(dry/sgridy))+nchop+3 ; c200
;
; now set up YGRID ZGRID
;
ygrid=fltarr(nygrid)
zgrid=fltarr(nzgrid)
ygrid=(findgen(nygrid)*sgridy)-(sgridy*float(nygrid-1)/2.0)
zgrid=(findgen(nzgrid)*sgridz)-(sgridz*float(nzgrid-1)/2.0)
;
if(iprint eq 1)then begin
print,'MAPSIMUL32: sgridy,sgridz,nygrid,nzgrid: ',sgridy,sgridz,nygrid,nzgrid
print,'MAPSIMUL32: ygrid:',ygrid
print,'MAPSIMUL32: zgrid:',zgrid
endif ; iprint
;
map=fltarr(nygrid,nzgrid)
mapn=map
maprms=map
mapdisp=map
mapoffy=map
mapoffz=map
;
; set up gaussian point source on natural grid SKY at centre of P32 field
; and print out source parameters and derived parameters like integrated
; flux density to infinity
;
sky=map
y0=float(nygrid-1)/2.0 ; source centre in units of natural grid
z0=float(nzgrid-1)/2.0 ; source centre
;
for iy=0,nygrid-1 do begin
y=(float(iy)-y0)*sgridy ; smallest numbers at lower left hand corner
for iz=0,nzgrid-1 do begin
z=(float(iz)-z0)*sgridz ; smallest numbers at lower left hand corner
mapoffy(iy,iz)=y
mapoffz(iy,iz)=z
d=y*y*ay+z*z*az
expmax=30.0
if(d gt expmax)then d=expmax
sky(iy,iz)=back+peak*exp(0.0-d)
endfor ; iz
endfor ; iy
;
; simulate observation
;
idry=round(dry/sgridy) ; s/c raster increment in units of natural map pixels
idumy=round(pixpitch/sgridy)
pixoff(0,*)=pixoff(0,*)/pixpitch ; pixel offsets in units of array pixel 
pixoff(0,*)=pixoff(0,*)*float(idumy) ;  pixel offsets in natural map pixels
idrz=round(drz/sgridz) ; s/c raster increment in units of natural map pixels
idumz=round(pixpitch/sgridz)
pixoff(1,*)=pixoff(1,*)/pixpitch ; pixel offsets in units of array pixel 
pixoff(1,*)=pixoff(1,*)*float(idumz) ;  pixel offsets in natural map pixels
;
if(array eq 'c100')then ycoffmax=3.0 ; offset of detector pixel from array centre
                                   ; in map sampling units
if(array eq 'c200')then ycoffmax=1.5 ; offset of detector pixel from array centre
                                   ; in map sampling units
if(array eq 'c100')then zcoffmax=pixpitch/sgridz ; offset of detector pixel from array centre
                                   ; in map sampling units
if(array eq 'c200')then zcoffmax=pixpitch/(sgridz*2.0) ; offset of detector pixel from array centre
                                   ; in map sampling units
;
for ipix=0,nmappix-1 do begin
ip=mappix(ipix)
ypoff=pixoff(0,ip) ; pixel offset from array centre in natural map pixels
zpoff=pixoff(1,ip)
for ipy=0,ny-1 do begin
yc=float(ipy)-float(ny-1)/2.0 ; array centre in units of s/c raster interval
yc=yc*float(idry) ; array centre in units of natural map pixel
yc=yc-ypoff ; pixel centre in units of natural map (should be close 
            ; to an integer
ycmin=0.0-float(ny-1)/2.0
ycmin=ycmin*float(idry)
ycmin=ycmin-ycoffmax ; minimum value; should be float(nchop/2)
yc=yc-ycmin+float(nchop/2)
iy1=round(yc)-nchop/2
iy2=iy1+nchop-1
;
for ipz=0,nz-1 do begin
zc=float(ipz)-float(nz-1)/2.0 ; array centre in units of s/c raster interval
zc=zc*float(idrz) ; array centre in units of natural map pixel
zc=zc-zpoff ; pixel centre in units of natural map (should be close 
            ; to an integer

zcmin=0.0-float(nz-1)/2.0
zcmin=zcmin*float(idrz)
zcmin=zcmin-zcoffmax ; minimum value; should be close to 0.0
zc=zc-zcmin
iz=round(zc)
;
for iy=iy1,iy2 do begin
map(iy,iz)=map(iy,iz)+flatsimul(ip)*sky(iy,iz)
map(iy,iz)=map(iy,iz)+darksimul(ip)
ss=randomn(seed,1)
ss=ss*rmssimul(ip)
map(iy,iz)=map(iy,iz)+ss(0)
mapn(iy,iz)=mapn(iy,iz)+1
maprms(iy,iz)=ss(0)
mapdisp(iy,iz)=ss(0)
endfor ; iy
endfor ; ipz
endfor ; ipy
endfor ; ipix
iii=where(mapn le 0,niii)
ii=where(mapn gt 0,nii)
if(niii eq 0)then map=map/mapn
if(niii gt 0)then map(ii)=map(ii)/mapn(ii)
;
if(iprint eq 1)then begin
print,'MAPSIMUL32: array, osamp,ny,nz = ',array,osamp,ny,nz
print,'MAPSIMUL32: nmappix,mappix = ',nmappix,mappix
print,'MAPSIMUL32: flatsimul:',flatsimul
print,'MAPSIMUL32: darksimul:',darksimul
print,'MAPSIMUL32: rmssimul: ',rmssimul
;
; if peak in units of Jy/sq arcsec and ay,az in arcsec^{-2}
modelsum=peak*3.14159/sqrt(ay*az)
print,'MAPSIMUL32 simulated background + central gaussian ',$
'has (prior to sampling):'
print,'back peak, ay, az, integrated flux:',back,peak,ay,az,modelsum
ii=where(mapn ge 1,nii)
print,'MAPSIMUL32 (statistics of sampled points):'
print,'min max total*pixelarea = ',$
min(map(ii)),max(map(ii)),total(map(ii))*sgridy*sgridz
;
endif ; iprint
;
return
end
