;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_GET_PARS      
;
; PURPOSE:
;       This routine sets up the drift fitting parameters in COMMON DETPARS
;       from the user entered values on 'INSPECT_SINGLE_PIXELS' GUI.
;
; DESCRIPTION:
;       The routine DRIFT32.PRO computes drift corrections using the values
;       in common block DETPARS as initial guesses. This routines fills up
;       the common block from user entered 4x6 matrix on 'Inspect signals per pixel'
;       GUI.
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_get_pars, pix, aval, bval, cval, params, cal
;
; INPUTS:
;       pix: IDL convention for selected pixel i.e pixel # - 1   
;       aval: A coefficients for parameters beta1, tau1, beta2, tau2
;       bval: B coefficients for parameters beta1, tau1, beta2, tau2
;       cval: C coefficients for parameters beta1, tau1, beta2, tau2
;       params: flags for aval, bval, cval, 1 = free, 0 = fixed        
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       cal: calibration type 
;       cal = 2 - drift fitting with user supplied free values (self calibration)
;       cal = 4 - drift fitting with user supplied fixed values
;          
;
; KEYWORDS:
;
; CALLED ROUTINES:
;
; CALLED FROM:    
;       PIA_ERD2MAP_SPIX      
;
; COMMON BLOCKS:
;
; PROCEDURE:
;       @common_all
;       @common_detpars
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;-
;-----------------------------------------------------------------------
;
PRO PIA_get_pars, pix, aval, bval, cval, params, cal

@common_detpars
@common_all   


        indx = WHERE(params EQ 1, cnt)
        IF cnt EQ 0 THEN  cal =4 $
        ELSE cal = 2
       
        IF (cal  EQ 4) THEN BEGIN

          betaoverride(pix) = FLOAT(STRTRIM(aval(0), 2))
          betacoeffoverride(pix) = FLOAT(STRTRIM(bval(0), 2))
          betaexpoverride(pix) = FLOAT(STRTRIM(cval(0), 2))

          tauoffsetoverride(pix) = FLOAT(STRTRIM(aval(1), 2))
          tauoverride(pix) = FLOAT(STRTRIM(bval(1), 2))
          gammaexpoverride(pix) = FLOAT(STRTRIM(cval(1), 2))
          

          betashootoverride(pix) = FLOAT(STRTRIM(aval(2), 2))
          betacoeffshootoverride(pix) = FLOAT(STRTRIM(bval(2), 2))
          betaexpshootoverride(pix) = FLOAT(STRTRIM(cval(2), 2))
          
          tauoffsetshootoverride(pix) = FLOAT(STRTRIM(aval(3), 2))
          taushootoverride(pix) = FLOAT(STRTRIM(bval(3), 2))
          gammaexpshootoverride(pix) = FLOAT(STRTRIM(cval(3), 2))

        ENDIF ELSE IF (cal  EQ 2) THEN BEGIN	
 
          betaoversc(pix) = FLOAT(STRTRIM(aval(0), 2))
          betacoeffoversc(pix) = FLOAT(STRTRIM(bval(0), 2))
          betaexpoversc(pix) = FLOAT(STRTRIM(cval(0), 2))

          tauoffsetoversc(pix) = FLOAT(STRTRIM(aval(1), 2))
          tauoversc(pix) = FLOAT(STRTRIM(bval(1), 2))
          gammaexpoversc(pix) = FLOAT(STRTRIM(cval(1), 2))
          
          betashootoversc(pix) = FLOAT(STRTRIM(aval(2), 2))
          betacoeffshootoversc(pix) = FLOAT(STRTRIM(bval(2), 2))
          betaexpshootoversc(pix) = FLOAT(STRTRIM(cval(2), 2))
          
          tauoffsetshootoversc(pix) = FLOAT(STRTRIM(aval(3), 2))
          taushootoversc(pix) = FLOAT(STRTRIM(bval(3), 2))
          gammaexpshootoversc(pix) = FLOAT(STRTRIM(cval(3), 2))


        ENDIF   
RETURN
END

