;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_SET_WPS       
;
; PURPOSE:
;       This routine initializes the Working parameter set (WPS) from the detector
;       default parameters.
;
; DESCRIPTION:
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_set_wps, vars, PIXL=pixl
;
; INPUTS:
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       vars(12, npix): 12 drift fitting parameters for all pixels          
;
; KEYWORDS:
;       pixl: IDL convention for selected pixel i.e pixel # - 1   
;             if this keyword is not defined that all pixel values
;             are changed in WPS otherwise only the ones for input
;             pixel.
;
; CALLED ROUTINES:
;        INITDETPARS
;
; CALLED FROM:    
;       PIA_ERD2MAP
;       PIA_ERD2MAP_SPIX      
;
; COMMON BLOCKS:
;
; PROCEDURE:
;       @common_detdefault
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;-
;-----------------------------------------------------------------------
;
PRO pia_set_wps, vars, pixl=pixl
@common_detdefault

      IF KEYWORD_set(pixl) then npixl = pixl $
      ELSE npixl = -1

      initdetpars

      IF npixl EQ -1 THEN BEGIN
 
          vars(0,*) = betadefault
          vars(1,*) = betacoeffdefault
          vars(2,*) = betaexpdefault

          vars(3,*) = tauoffsetdefault
          vars(4,*) = taudefault
          vars(5,*) = gammaexpdefault

          vars(6,*) = betashootdefault
          vars(7,*) = betacoeffshootdefault
          vars(8,*) = betaexpshootdefault

          vars(9,*) = tauoffsetshootdefault
          vars(10,*) = taushootdefault
          vars(11,*) = gammaexpshootdefault

      ENDIF ELSE IF npixl GE 0  THEN BEGIN

          vars(0,npixl) = betadefault(npixl)
          vars(1,npixl) = betacoeffdefault(npixl)
          vars(2,npixl) = betaexpdefault(npixl)

          vars(3,npixl) = tauoffsetdefault(npixl)
          vars(4,npixl) = taudefault(npixl)
          vars(5,npixl) = gammaexpdefault(npixl)

          vars(6,npixl) = betashootdefault(npixl)
          vars(7,npixl) = betacoeffshootdefault(npixl)
          vars(8,npixl) = betaexpshootdefault(npixl)

          vars(9,npixl) = tauoffsetshootdefault(npixl)
          vars(10,npixl) = taushootdefault(npixl)
          vars(11,npixl) = gammaexpshootdefault(npixl)
      ENDIF

RETURN

END
	
	
	
	
