;+
; NAME:
;	pr_open
; PURPOSE:
;	Select a printer queue or a filename.
;	Open the device for succeeding printing, drawing, etc.
;	The device should be closed afterwords by 'pr_close'
; CATEGORY:
;	PRINTER
; CALLING SEQQUENCE:
;	pr_open, group=group
; ARGUMENTS:
;	-
; KEYWORDS:
;	group		(input, optional)
;			group_leader of widgets
; CALLED ROUTINES:
;	-
; COMMON BLOCKS:
;	common printer
; RESTRICTIONS:
;	This routine is restricted to the local computing environment
;	on the following reasons:
;	- local names of printer queues are defined in this routine.
;	  An adaption to any site is straight forward.
; REVISION HISTORY:
;	16-feb-94 ds	created: D. Skaley, MPIK Heidelberg
;-


;*
; pr_fq
; prepare format and queue list widget contents
;*

pro pr_fq, par, ixf, ixq, fnames, qnames

	common printer, pstatus, pformats, pqueues

	if par.color $
		then ixf = where(pformats.color eq 1) $
		else ixf = indgen(n_elements(pformats))
	fnames = pformats(ixf).name
	if (where(ixf eq par.ixf))(0) eq -1 then par.ixf = ixf(0)
	ixq = where(pqueues.device eq pformats(par.ixf).device)
	if ixq(0) ne -1 then begin
		queues = pqueues(ixq)
		if par.color then ixq = ixq(where(queues.color eq 1))
	endif
	if ixq(0) eq -1 then begin
		qnames = ''
		par.ixq = -1
	endif else begin
		qnames = queues(ixq).name
		if (where(ixq eq par.ixq))(0) eq -1 then par.ixq = ixq(0)
	endelse

	return
	end

;*
; pr_open_event:
; event handling routine for pr_open
;*

PRO pr_open_event, event

	common printer, pstatus, pformats, pqueues

	widget_control, event.id, get_uvalue = action
	if n_elements(action) eq 0 then return
	widget_control, event.top, get_uvalue = pid
	widget_control, pid.first, get_uvalue = tmp, /no_copy
	par = tmp.par
	ixf = tmp.ixf
	ixq = tmp.ixq

	case action of
	'FORM':	begin
		par.ixf = ixf(event.index)
		pr_fq, par, ixf, ixq, fnames, qnames
		goto,set_format_and_queue
		end
	'QUEU':	begin
		par.ixq = ixq(event.index)
		pr_fq, par, ixf, ixq, fnames, qnames
		goto,set_format_and_queue
		end
	'FSEL':	begin
		par.fileselect = event.select
		widget_control, pid.filename, sensitive = event.select
		pr_fq, par, ixf, ixq, fnames, qnames
		goto,set_format_and_queue
		end
	'BW':	begin
		par.color = 0
		pr_fq, par, ixf, ixq, fnames, qnames
		goto,set_format_and_queue
		end
	'COL':	begin
		par.color = 1
		pr_fq, par, ixf, ixq, fnames, qnames
		goto,set_format_and_queue
		end
	'PORT':	par.landscape = 0
	'LAND':	par.landscape = 1
	'COPY':	par.copy = event.select
	'INTP':	par.interpolate = event.select
	'OK':	begin

		; save status for pr_close and for next call of pr_open
		pstatus.landscape = par.landscape
		pstatus.color = par.color
		pstatus.ixf = par.ixf
		pstatus.fileselect = par.fileselect
		pstatus.copy = par.copy
		pstatus.interpolate = par.interpolate

		; determine device and filename
		new_device = pformats(par.ixf).device
		if par.fileselect then begin
			pstatus.queue = ''
			widget_control, pid.filename, get_value = i
			pstatus.filename = i(0)
			if pstatus.filename eq '' then begin
				pstatus.filename = 'idl.' + new_device
				case !version.os of
				'vms':	pstatus.filename = $
					'sys$scratch:' + pstatus.filename
				else:
				endcase
			endif
		endif else begin
			if par.ixq eq -1 then begin
				message,/info,'No queue selected'
				widget_control, pid.first, $
				set_uvalue={ par:par, ixf:ixf, ixq:ixq }, $
					/no_copy
				return
			endif
			pstatus.queue = strtrim(pqueues(par.ixq).name)
			pstatus.filename = 'idl.' + new_device
			case !version.os of
			'vms':	pstatus.filename = $
				'sys$scratch:' + pstatus.filename
			else:
			endcase
			pstatus.ixq = par.ixq
		endelse

		; save screen device and switch to print device
		pstatus.prev_device = !d.name
		set_plot, new_device, $
			copy = par.copy, $
			interpolate = par.interpolate

		; set device characteristics
		if new_device eq 'PS' then begin
		   if pstatus.landscape then begin
			device, filename = pstatus.filename, $
			color = pstatus.color, $
			bits_per_pixel = 8, $
			/landscape
		   endif else begin
			device, filename = pstatus.filename, $
			color = pstatus.color, $
			bits_per_pixel = 8, $
			/portrait
		   endelse
		endif else begin
		   if pstatus.landscape then begin
			device, filename = pstatus.filename, /landscape
		   endif else begin
			device, filename = pstatus.filename, /portrait
		   endelse
		endelse

		; indicate that print file is open
		pstatus.active = 1

		widget_control, /destroy, event.top
		return
		end

	'CANC':	begin
		widget_control, /destroy, event.top
		return
		end
	else:
	endcase
	widget_control, pid.first, $
		set_uvalue={ par:par, ixf:ixf, ixq:ixq }, /no_copy
	return

set_format_and_queue:
	widget_control, pid.formats, set_value = fnames
	widget_control, pid.queues, set_value = qnames
	widget_control, pid.selformat, set_value = pformats(par.ixf).name
	if par.fileselect or par.ixq eq -1 $
		then pr = "" $
		else pr = pqueues(par.ixq).name
	widget_control, pid.selqueue, set_value = pr
	widget_control, pid.first, $
		set_uvalue={ par:par, ixf:ixf, ixq:ixq }, /no_copy
	return

	end

;*
; pr_open
; main program to create widget
;*

PRO pr_open, group=group

	if xregistered("pr_open") then return

	common printer, pstatus, pformats, pqueues

; initialize structures if necessary

	if n_elements(pstatus) eq 0 then begin

		pstatus = { $
		 	ixf:0,		$
			ixq:0,		$
			fileselect:0,	$
			filename:"",	$
			landscape:0,	$
			color:1,	$
			copy:1,		$
			interpolate:1,	$
			active:0,	$
			queue:"",	$
			prev_device:""	$
		}

		dummy = { pformat, device:"", name:"", color:0 }
		dummy = { pqueue, device:"", name:"", color:0 }

		pformats = [	$
			{ pformat, "PS", "Postscript", 1 },	$
			{ pformat, "PCL", "PCL-5", 0 },		$
			{ pformat, "HP", "HP-GL", 0 },		$
			{ pformat, "LJ", "LJ250", 0 },		$
			{ pformat, "REGIS", "REGIS", 0 },	$
			{ pformat, "TEK", "Tektronix", 0 },	$
			{ pformat, "CGM", "CGM", 0 }		$
		]
		pqueues = [	$
			{ pqueue, "PS", "GE$LN_16", 0 },	$
			{ pqueue, "PS", "RZ$HP_33", 0 },	$
			{ pqueue, "PS", "GE$HP_34", 0 },	$
			{ pqueue, "PS", "GE$HP_35", 0 },	$
			{ pqueue, "PS", "BO$SV_90", 0 },	$
			{ pqueue, "PS", "GE$HP_1200", 1 },	$
			{ pqueue, "PS", "BO$CN_2", 1 },		$
			{ pqueue, "PCL", "GE$HP_1200", 0 }	$
		]

		pstatus.active = 0	; initial state: no printing active

	endif

; do nothing if printing was already activated

	if pstatus.active then begin
		message,/info,'printing already active: call pr_close.'
		return
	endif

; initialize temporary settings for this call of pr_open

	par = { $
		ixf:pstatus.ixf,		$
		ixq:pstatus.ixq,		$
		fileselect:pstatus.fileselect,	$
		filename:pstatus.filename,	$
		landscape:pstatus.landscape,	$
		color:pstatus.color,		$
		copy:pstatus.copy,		$
		interpolate:pstatus.interpolate	$
        }
	if not par.fileselect then par.filename=''

; widget ids used in event handler

	pid = { $
		first:0L,	$
		formats:0L,	$
		queues:0L,	$
		selformat:0L,	$
		selqueue:0L,	$
		filename:0L	$
	}

; prepare contents of format and queue list widgets

	pr_fq, par, ixf, ixq, fnames, qnames

; define widget

	id_base = widget_base( title="Printer Selection", /column )
	pid.first = widget_base( id_base, /row )

	id_left = widget_base( pid.first, /column )

	 id_2 = widget_base( id_left, /column, /exclusive, /frame )
	  id_3 = widget_button( id_2, value="Black & White", uvalue='BW', $
				/no_release )
	  widget_control, id_3, set_button=(1-par.color)
	  id_3 = widget_button( id_2, value="Color", uvalue='COL', $
				/no_release )
	  widget_control, id_3, set_button=par.color

	 id_2 = widget_base( id_left, /column, /exclusive, /frame )
	  id_3 = widget_button( id_2, value="Portrait", uvalue='PORT', $
				/no_release )
	  widget_control, id_3, set_button=(1-par.landscape)
	  id_3 = widget_button( id_2, value="Landscape",uvalue='LAND', $
				/no_release )
	  widget_control, id_3, set_button=par.landscape

	 id_2 = widget_base( id_left, /column, /nonexclusive, /frame )
	  id_3 = widget_button( id_2, value="Copy", uvalue='COPY' )
	  widget_control, id_3, set_button=par.copy
	  id_3 = widget_button( id_2, value="Interpolate",uvalue='INTP' )
	  widget_control, id_3, set_button=par.interpolate

	id_2 = widget_base( id_left, /column )
	  id_3 = widget_button( id_2, value='OK', uvalue='OK' )
	  id_3 = widget_button( id_2, value='Cancel', uvalue='CANC' )

	id_right = widget_base( pid.first, /column )

	id_1 = widget_base( id_right, /row )
	 id_2 = widget_base( id_1, /column )
	  id_3 = widget_label( id_2, value="        Formats        " )
	  pid.formats = widget_list( id_2, ysize=4, val=fnames, uval='FORM')
	  id_3 = widget_label( id_2, value="selected:       " )
	  pid.selformat = widget_label( id_2, value=pformats(par.ixf).name )
	 id_2 = widget_base( id_1, /column )
	  id_3 = widget_label( id_2, value="        Printers        " )
	  pid.queues = widget_list( id_2, ysize=4, val=qnames, uval='QUEU')
	  id_3 = widget_label( id_2, value=" " )
	  if par.fileselect or par.ixq eq -1 $
		then pr = "" $
		else pr = pqueues(par.ixq).name
	  pid.selqueue = widget_label( id_2, value=pr )

	id_1 = widget_base( id_right, /column, /frame )
	 id_2 = widget_base( id_1, /nonexclusive, /column )
	  id_3 = widget_button( id_2, value="print to file", uvalue='FSEL' )
	  widget_control, id_3, set_button=par.fileselect
	 pid.filename = widget_text( id_1, /editable, xsize=24, $
			value=par.filename )
	 widget_control, pid.filename, sensitive=par.fileselect

; realize and register widget

	widget_control, pid.first, $
		set_uvalue={ par:par, ixf:ixf, ixq:ixq }, /no_copy
	widget_control, id_base, /realize, set_uvalue=pid
	xmanager, 'pr_open', id_base, /modal, group=group

	return
	end
