pro relativise,badprev,nrelmin,k1,k2,pixplt
;
; find relativisation factors (array rel) from s_in ignoring
; bad data points in badprev. s is created from s_in by implementing
; relativisation factors. rms of data is not taken into account by fit.
; Modified May 2000 so as to only give an offset correction derived from
; the median value of illumination. The former version of the program was 
; copied to relativise_former.pro
;
; input:
;         s_in(npix,nptin) Vs-1 (=V(i+1)-V(i))
;         rs(nptin) readout cycle (eg  1 2 3 4 1 .... for ninramp=4)
;         nch there are nch+2 chopper transitions
;         NRELMIN is the minimum reads of each read class on a chopper plateau
;         for the chopper plateau to be included in the relativisation
;         calculation
;         jf=lonarr(nch+2,ninchmax,ninramp-1) denotes identifiers of
;         all combinations of chopper plateau and readout class.
;         NJF(nch+2,ninramp-1) is the number of reads of a given readout class 
;         in a given chopper plateau
;         NINCHMAX maximum number of reads of a given readout class in a given
;          chopper plateau
;         IRELSTART; the relativisation is scaled to the average signal of the
;         done with respect to the means of the signals from the read classes 
;         IRELSTART....NINRAMP-IRELSTART (NOT IDL convention).
;         where the first signal (V/s) is read class 1 
;         and the last NINRAMP-2. Typically for a P32 with ninramp=4 
;         IRELSTART should be 2.
;         k1 and k2 are print/plot range for diagnostic output
;         pixplt: data from pixel pixplt will be printed
;
; output:
;         rel(npix,2,ninramp-1): relativisation factors scorr=a+b*s
;         s(npix,nptin) Vs-1 (=V(i+1)-V(i)) relativised
;
; note no weighting with rms is desirable in relativise as high rms points
; may indicate high signals with drifts
;
; the relativisation is scaled to the average signal of the
; done with respect to the means of the signals from the read classes 
; IRELSTART....NINRAMP-IRELSTART (NOT IDL convention).
; where the first signal (V/s) is eread class 1 and the 
; last NINRAMP-2. Typically for a P32 with ninramp=4 IRELSTART should be 2.
;
;print,' '
;kk2=abs(k2) ; printing parameters
;kk1=abs(k1)
;
; for each chopper plateau find median of signals in s_in for each of the 
; ninramp-1 ndrs, ignoring data marked in badprev. Also set up flag array
; BADSIGMED to flag which plateaus should be ignored in the calculation
; of the relativisation factors REL due to there being too many points
; in that plateau flagged by BADPREV for any one of the read classes.
;
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;
@common_steerin
@common_steer1
@common_signal
;
sigmed=dblarr(npix,nch+2,ninramp-1)
badsigmed=lonarr(npix,nch+2,ninramp-1)
for ipix=0L,npix-1 do begin
print,'RELATIVISE: computing statistics for pixel',ipix+1
;good=double((badprev(ipix,*))
;good(*)=1.0
;good(where(badprev(ipix,*) ne 0) = 0.0
for ichop=-1L,nch do begin ; -1 to nch range is to include first and last too
;sigmed(ipix,0,ird)=median(s_in(ipix,jf(0,0:njf(0,ird)-1,ird)))
;sigmed(ipix,nch+1,ird)=median(s_in(ipix,jf(nch+1,0:njf(nch+1,ird)-1,ird)))
for ird=0L,ninramp-2 do begin
badsigmed(ipix,ichop+1,ird)=0L ; reset since we may be on a second iteration
nijf=njf(ichop+1,ird) ;
if(nijf le 0)then print,'RELATIVISE: warning readout data probably not correct'
if(nijf gt 0)then begin ; loop added rjt jan00
ijf=jf(ichop+1,0:nijf-1,ird)
jfbad=badprev(ipix,ijf)
s_sub=s_in(ipix,ijf)
jfsub=where(jfbad eq 0,njfgood)
;if(ipix eq 1 and ichop eq -1 and ird eq 0)then begin
;print,'RELATIVISE:'
;help,s_sub
;help,ijf
;help,jfsub
;endif ; help
if(njfgood ge nrelmin)then begin
evenodd,njfgood,iodd
;evenodd,nijf,iodd
if(iodd eq 0)then begin ; avoid bias for medians of small even number of data
ss=s_sub(jfsub)
ss=ss(sort(ss))
n1=njfgood/2
n2=n1-1
;if(ipix eq 0 and ichop eq 11 and ird eq 1)then begin
;print,'RELATIVISE:'
;print,'ipix,ichop,ird: ',ipix,ichop,ird
;print,'rd(ijf): ',rd(ijf)
;help,s_sub
;print,s_sub
;help,ijf
;print,ijf
;help,nijf
;print,nijf
;help,jfsub
;print,jfsub
;help,jfbad
;print,jfbad
;help,ss
;print,ss
;help,n1
;print,n1
;endif ; help
;
dummy=(ss(n1)+ss(n2))/2.0
endif ; iodd
if(iodd eq 1)then begin ; normal median for odd number of data
dummy=median(s_sub(jfsub))
endif ; iodd
sigmed(ipix,ichop+1,ird)=dummy
endif ; nrelmin
if(njfgood lt nrelmin)then begin
badsigmed(ipix,ichop+1,ird)=1L
endif ; nrelmin
endif ; nijf gt 0 loop added rjt jan00
endfor ; ird
endfor ; ichop
endfor ; ipix
;
; now find ratios between read classes ird=0K,ninramp-2 and the average
; of irelstart-1 : ninramp-2 as a function average of 
; signal = irelstart-1 : ninramp-2
; and fit this sequence for each pixel with a + b * signal, ignoring
; plateaus with badsigmed = 1.
;
print,'RELATIVISE: total(badsigmed) = ',total(badsigmed)
y=dblarr(nch+2,ninramp-1)
x=dblarr(nch+2)
for ipix=0L,npix-1 do begin
y(*,*)=0.0
x(*)=0.0
;
;if(ipix eq 0)then begin
;window,10,xs=400,ys=400,tit='read 1/0 versus time'
;stest=s_in(where(rs eq 2))-s_in(where(rs eq 2)-1)
;plot,stest(0:100)
;window,10,xs=400,ys=400,tit='read 1 versus read 0'
;print,'RELATIVISE: plotting pixel ipix:'
;plot,sigmed(ipix,*,0),sigmed(ipix,*,1),psym=1
;oplot,[0,0],[10,10]
;window,11,xs=400,ys=400,tit='read 2 versus read 0'
;plot,sigmed(ipix,*,0),sigmed(ipix,*,2),psym=1
;oplot,[0,0],[10,10]
;endif
;
for ichop=-1L,nch do begin ; -1 to nch range is to include first and last too
x(ichop+1)=total(sigmed(ipix,ichop+1,irelstart-1:ninramp-irelstart-1))$
/double(ninramp-2*irelstart+1) ; ie miss out irelstart-1 from start of ramp and
                               ; from end of ramp
for ird=0L,ninramp-2 do begin
y(ichop+1,ird)=sigmed(ipix,ichop+1,ird)
;if(x(ichop+1) eq 0.0)then $
;print,'RELATIVISE: **** zero x(ichop+1) = ',x(ichop+1)
;if(ipix eq 5 and ichop gt 100 and ichop lt 110)then begin
;print,'RELATIVISE: ipix,ichop,ird,x(ichop+1),sigmed(ipix,ichop+1,ird):'
;print,ipix,ichop,ird,x(ichop+1),sigmed(ipix,ichop+1,ird),y(ichop+1,ird)
;endif
endfor ; ird
endfor ; ichop

;
dummy=lonarr(nch+2)
for ird=0L,ninramp-2 do begin
dummy(*)=badsigmed(ipix,*,ird)
;help,dummy
idum=where(dummy eq 0) ; don't include plateaus with insufficient statistics
;
; plot sigmed(ipix,*,ird)-sigmed(ipix,*,ird) to se iff the ratio is constant
;
;if(ird eq 1 and ipix eq 1) then begin
;window,15
;plot,(sigmed(ipix,*,ird)-sigmed(ipix,*,ird-1))
;endif
;
;help,idum
aa=poly_fit(x(idum),y(idum,ird),1)
rel(ipix,0:1,ird)=aa(*)
;
; modification added May 2000
;
;if(ird eq 0 and ipix eq 0)then save,ird,x,y,idum,/xdr,file='relativise.xdr'
xmed=median(x(idum))
imed=where(float(x) eq xmed,nimed)
ymed=median(y(idum,ird))
yoffset=xmed-ymed
rel(ipix,0,ird)=0.0-yoffset
rel(ipix,1,ird)=1.0
; end modification
;
;print,'RELATIVISE: ipix,ird,rel(ipix,0:1.ird):',ipix,ird,rel(ipix,0:1,ird)
iqw=where(rs eq (ird+1)) ; ie set all signals (also those flagged bad)
s(ipix,iqw)=(0.0-rel(ipix,0,ird)+s_in(ipix,iqw))/rel(ipix,1,ird)
; modification may 2000
;s(ipix,iqw)=s_in(ipix,iqw)+yoffset
endfor ;ird
;
endfor ; ipix
;window,10,xs=400,ys=400,tit='read 1/0 versus time'
;stest=s_in(where(rs eq 2))-s_in(where(rs eq 2)-1)
;plot,stest(0:100)
;window,10,xs=400,ys=400,tit='read 1 versus read 0'
;print,'RELATIVISE: plotting pixel 5:'
;plot,sigmed(5,*,0),sigmed(5,*,1),psym=1
;oplot,[0,0],[10,10],linestyle=0
;window,11,xs=400,ys=400,tit='read 2 versus read 0'
;plot,sigmed(5,*,0),sigmed(5,*,2),psym=1
;oplot,[0,0],[10,10],linestyle=1
;
return
end


