;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       SAVE_TEMP_PARS    
;
; PURPOSE:
;       This routine reads the values of current drift fitting parameters 
;       and flags from 4x6 matrix shown on 'INSPECT_SINGLE_PIXELS' GUI, and
;       translates it into temporary working set of (12, npix) arrays of values
;       and flags.
;
; DESCRIPTION:
;       The routine reads the text widgets for the values of 'A', 'B', 'C' for
;       beta1, tau1, beta2, and tau2, and the flag buttons for these parameters.
;       The values are then written into (12, npix) arrays.     
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       save_temp_pars, lid, pvars, params
;
; INPUTS:
;       lid: IDL structure containing the widget ids of paramters and pixel info
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       pvars(12, npix): current values of detector modal parameters
;       params(12, npix): current values of detector modal parameters flags, 
;                         Free (1) or Fix (0).
;          
; KEYWORDS:
;       None.
;
; CALLED ROUTINES:
;
; CALLED FROM:    
;       PIA_ERD2MAP_SPIX      
;
; COMMON BLOCKS:
;
; PROCEDURE:
;       None.
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;-
;-----------------------------------------------------------------------
;
 PRO save_temp_pars, lid, pvars, params
 
       debug = 0
       avar = lid. avar
       bvar = lid. bvar
       cvar = lid. cvar

       FOR i = 0,3 DO BEGIN
          WIDGET_CONTROL, lid.a(i), GET_VALUE = tmp
          avar(i) = tmp(0)
          WIDGET_CONTROL, lid.b(i), GET_VALUE = tmp
          bvar(i) = tmp(0)
          WIDGET_CONTROL, lid.c(i), GET_VALUE = tmp
          cvar(i) = tmp(0)
       ENDFOR
       vtmps = [avar(0), bvar(0), cvar(0), $
                avar(1), bvar(1), cvar(1), $
                avar(2), bvar(2), cvar(2), $
                avar(3), bvar(3), cvar(3)]   

       ptmps = [lid.varsa(0), lid.varsb(0), lid.varsc(0), $
               lid.varsa(1), lid.varsb(1), lid.varsc(1), $
               lid.varsa(2), lid.varsb(2), lid.varsc(2), $
               lid.varsa(3), lid.varsb(3), lid.varsc(3)]

       np = lid.pix
       IF debug THEN print,'saving pixel #',np
       pvars(*, np-1) = vtmps
       params(*, np-1) = ptmps
       

      
RETURN
END

