pro setrange,imodel,tauwk,sillmin,sillmax,slimit,ilower
;
; For some models (currently only imodel=3) the shooting detector time 
; constant is a strong function of brightness and can become negative,
; which we deem to be unphysical. The routine SETRANGE finds the threshold
; illumination SLIMIT above or below which the detector time constant is 
; negative. The value of SLIMIT passed to the calling routine DRIFTCORR can 
; be used to prevent solutions with illuminations the wrong side of SLIMIT 
; requiring a negative shooting time constant.
;
; input: 
;       - IMODEL (to use as a check)
;       - SILLMIN,SILLMAX the range of illuminations for which a solution
;                         is to be sought by DRIFTCORR
;                TAUWK, the long time constant called by EXPSHOOT 
;                (not in common)
; output:
;       - SLIMIT (V/s) if there is an illumination between SILLMIN and
;                      SILLMAX for which the shooting time constant is zero,
;                      SLIMIT is set to this threshold. Otherwise,
;                      SLIMIT is set to -999.0
;       - ILOWER       = 1 if SLIMIT is to be treated as a lower limit
;                         (ie for illuminationss < SLIMIT negative time const.)
;                      = 0 if SLIMIT is to be treated as an upper limit
;                         (ie for illuminations > SLIMIT negative time const.)
;
if(imodel ne 3)then begin
print,'Warning - at present SETRANGE should only be called for imodel=3'
goto,c999
endif ; imodel ne 3
;
n=10000
test=sillmin+findgen(n)*(sillmax-sillmin)/float(n-1)
setshoottime_model3,tauwk,test,testlong,testshoot
;
atestshoot=abs(testshoot)
;
i=where(atestshoot eq min(atestshoot),ni)
;
slimit=-999.0
ilower=1
;
i=i(0)
if(i ne n-1 and i ne 0)then begin ; we don't care if limits don't match bounds
;
if(testshoot(i+1)*testshoot(i-1) le 0.0)then begin 
; threshold for min. shooting time constant is at or betw. SILLMIN and SILLMIN
slimit=test(i)
if(testshoot(i+1) lt testshoot(i))then ilower=0
if(testshoot(i+1) gt testshoot(i))then ilower=1
;
endif ; testshoot(i+1)*testshoot(i-1) le 0.0
;
endif ; i ne n-1 and i ne 0
;
c999:
;
return
end






