pro setshoottime_model3,tauwk,sillum,tlong,tshoot
;
; Finds detector time constants TLONG and TSHOOT as a function of 
; illumination SILLUM for imodel=3 (imodel eq 3) 
; This is needed by the routine EXPSHOOT and SETRANGE, so should be 
; defined in one place (this routine). When called by SETRANGE SILLUM and
; the output TLONG TSHOOT can be arrays, as needed by SETRANGE. Where called 
; by SOLVEILLUM1, SOLVEBACK2, however, only scalars are input.
; Hence the trap for tlong and tshoot (below) only uses sillum(0),
; to maintain speed.
;
; input:
;       - common MOREPAR with shooting detector parameters and the zero bias
;                TAUWK, the long time constant called by EXPSHOOT 
;                (not in common)
;       - SILLUM illumination in V/s (single vlue or array)
;
; output:
;       - TSHOOT, the exponential time constant for shooting, in s; same
;                 dimension as SILLUM
;       - TLONG,  the exponential time constant for the long response, in s;
;                 same dimension as SILLUM
;
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp ; for array
@common_morepar
;common morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
;
em=abs(sillum-zerobias)+zerobias
eg=em^gammaexp
egs=em^gammaexpshoot
tlong=tauoffset+(tauwk/eg)
tshoot=tauoffsetshoot+(taushoot/egs)
tlongmax=300000.0 ; turn off by etting to a big number
if(array eq 'c100')then begin
ii=where(tlong gt tlongmax,nii)
if(nii ge 1)then tlong(ii)=tlongmax 
endif ; array eq 'c100'
;
;if(tlong(0) lt 0.0)then print,'*** tlong < 0: sillum(0),tlong(0) = ',$
;sillum(0),tlong(0)
;if(tshoot(0) lt 0.0)then print,'*** tshoot < 0: sillum(0),tshoot(0) = ',$
;sillum(0),tshoot(0)
;
if(tlong(0) lt 0.0)then tlong(0)=0.0
if(tshoot(0) lt 0.0)then tshoot(0)=0.0
;
return
end







